@@ -38,7 +38,7 @@
 
 void github_print_checks(struct github_check_list const *checks);
 void github_print_checks(struct github_check_list const *const list);
-int github_checks(char const *const owner, char const *const repo,
+int github_checks(struct gcli_path const *const repo_path,
                   char const *const ref, int const max);
 int subcommand_ci(int argc, char *argv[]);
 
