@@ -1,5 +1,5 @@
 /*
- * Copyright 2021, 2022 Nico Sonack <nsonack@herrhotzenplotz.de>
+ * Copyright 2021-2025 Nico Sonack <nsonack@herrhotzenplotz.de>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -34,40 +34,42 @@
 #include <config.h>
 #endif
 
-#include <sn/sn.h>
 #include <gcli/gcli.h>
 
-#ifdef HAVE_SYS_QUEUE_H
+#include <gcli/port/sv.h>
+
 #include <sys/queue.h>
-#endif /* HAVE_SYS_QUEUE_H */
 
 struct gcli_config_entry {
 	TAILQ_ENTRY(gcli_config_entry) next;
-	sn_sv key;
-	sn_sv value;
+	gcli_sv key;
+	gcli_sv value;
 };
 
 TAILQ_HEAD(gcli_config_entries, gcli_config_entry);
 
 int gcli_config_parse_args(struct gcli_ctx *ctx, int *argc, char ***argv);
 int gcli_config_init_ctx(struct gcli_ctx *ctx);
-void gcli_config_get_upstream_parts(struct gcli_ctx *ctx, sn_sv *owner, sn_sv *repo);
+void gcli_config_get_upstream_parts(struct gcli_ctx *ctx, gcli_sv *owner, gcli_sv *repo);
 char *gcli_config_get_apibase(struct gcli_ctx *);
-sn_sv gcli_config_find_by_key(struct gcli_ctx *ctx, char const *section_name,
-                              char const *key);
+gcli_sv gcli_config_find_by_key(struct gcli_ctx *ctx, char const *section_name,
+                                char const *key);
 
 char *gcli_config_get_editor(struct gcli_ctx *ctx);
 char *gcli_config_get_token(struct gcli_ctx *ctx);
 char *gcli_config_get_pager(struct gcli_ctx *ctx);
+char *gcli_config_get_url_open_program(struct gcli_ctx *ctx);
 char *gcli_config_get_account_name(struct gcli_ctx *ctx);
-sn_sv gcli_config_get_upstream(struct gcli_ctx *ctx);
-sn_sv gcli_config_get_base(struct gcli_ctx *ctx);
+gcli_sv gcli_config_get_upstream(struct gcli_ctx *ctx);
+gcli_sv gcli_config_get_base(struct gcli_ctx *ctx);
 gcli_forge_type gcli_config_get_forge_type(struct gcli_ctx *ctx);
-sn_sv gcli_config_get_override_default_account(struct gcli_ctx *ctx);
+gcli_sv gcli_config_get_override_default_account(struct gcli_ctx *ctx);
 bool gcli_config_pr_inhibit_delete_source_branch(struct gcli_ctx *ctx);
 int gcli_config_get_repo(struct gcli_ctx *ctx, char const **, char const **);
-int gcli_config_have_colours(struct gcli_ctx *ctx);
-int gcli_config_display_progress_spinner(struct gcli_ctx *ctx);
+int gcli_config_get_remote(struct gcli_ctx *ctx, char const **remote);
+bool gcli_config_have_colours(struct gcli_ctx *ctx);
+bool gcli_config_display_progress_spinner(struct gcli_ctx *ctx);
+bool gcli_config_render_markdown(struct gcli_ctx *ctx);
 bool gcli_config_enable_experimental(struct gcli_ctx *ctx);
 struct gcli_config_entries const *gcli_config_get_section_entries(
 	struct gcli_ctx *ctx, char const *section_name);
