@@ -1,5 +1,5 @@
 /*
- * Copyright 2023 Nico Sonack <nsonack@herrhotzenplotz.de>
+ * Copyright 2023-2025 Nico Sonack <nsonack@herrhotzenplotz.de>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -36,6 +36,10 @@
 
 #include <gcli/issues.h>
 
+#include <gcli/cmd/actions.h>
+
+extern struct gcli_cmd_actions gcli_issue_actions;
+
 void gcli_print_issues(enum gcli_output_flags const flags,
                        struct gcli_issue_list const *const list, int const max);
 
