@@ -35,16 +35,14 @@
 #include <gcli/gitlab/pipelines.h>
 
 void gitlab_print_pipelines(struct gitlab_pipeline_list const *const list);
-int gitlab_pipelines(char const *owner, char const *repo, int const count);
 
-int gitlab_mr_pipelines(char const *owner, char const *repo, int const mr_id);
+int gitlab_mr_pipelines(struct gcli_path const *const path);
+
+int gitlab_pipeline_jobs(struct gcli_path const *const path, int count);
 
-int gitlab_pipeline_jobs(char const *owner, char const *repo, long pipeline,
-                         int count);
 void gitlab_print_jobs(struct gitlab_job_list const *const list);
 
 void gitlab_print_job_status(struct gitlab_job const *const job);
-int gitlab_job_status(char const *owner, char const *repo, long const jid);
 
 int subcommand_pipelines(int argc, char *argv[]);
 
