@@ -36,6 +36,8 @@
 
 #include <gcli/gcli.h>
 
+#include <time.h>
+
 enum {
 	DATEFMT_ISO8601,
 	DATEFMT_GITLAB,
@@ -44,4 +46,9 @@ enum {
 int gcli_normalize_date(struct gcli_ctx *ctx, int fmt, char const *const input,
                         char *output, size_t const output_size);
 
+int gcli_parse_iso8601_date_time(struct gcli_ctx *ctx, char const *input,
+                                 time_t *out);
+
+int gcli_format_as_localtime(struct gcli_ctx *ctx, time_t timestamp, char **out);
+
 #endif /* GCLI_DATE_TIME_H */
