@@ -1,5 +1,5 @@
 /*
- * Copyright 2021, 2022 Nico Sonack <nsonack@herrhotzenplotz.de>
+ * Copyright 2021-2025 Nico Sonack <nsonack@herrhotzenplotz.de>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -40,6 +40,7 @@
 #include <gcli/issues.h>
 #include <gcli/labels.h>
 #include <gcli/milestones.h>
+#include <gcli/path.h>
 #include <gcli/pulls.h>
 #include <gcli/releases.h>
 #include <gcli/repos.h>
@@ -48,7 +49,7 @@
 
 /* Hopefully temporary hack */
 typedef int (*gcli_get_pull_checks_cb)(
-	struct gcli_ctx *, char const *, char const *, gcli_id,
+	struct gcli_ctx *, struct gcli_path const *,
 	struct gcli_pull_checks_list *);
 
 /**
@@ -59,32 +60,36 @@ struct gcli_forge_descriptor {
 	 * Submit a comment to a pull/mr or issue */
 	int (*perform_submit_comment)(
 		struct gcli_ctx *ctx,
-		struct gcli_submit_comment_opts opts);
+		struct gcli_submit_comment_opts const *opts);
 
 	/**
 	 * List comments on the given issue */
 	int (*get_issue_comments)(
 		struct gcli_ctx *ctx,
-		char const *owner,
-		char const *repo,
-		gcli_id issue,
+		struct gcli_path const *issue_path,
 		struct gcli_comment_list *out);
 
 	/**
 	 * List comments on the given PR */
 	int (*get_pull_comments)(
 		struct gcli_ctx *ctx,
-		char const *owner,
-		char const *repo,
-		gcli_id pr,
+		struct gcli_path const *pull_path,
 		struct gcli_comment_list *out);
 
+	/**
+	 * Get a specific comment on an issue or pull request with the given ID */
+	int (*get_comment)(
+		struct gcli_ctx *ctx,
+		struct gcli_path const *target,
+		enum comment_target_type target_type,
+        	gcli_id comment_id,
+		struct gcli_comment *out);
+
 	/**
 	 * List forks of the given repo */
 	int (*get_forks)(
 		struct gcli_ctx *ctx,
-		char const *owner,
-		char const *repo,
+		struct gcli_path const *repo_path,
 		int max,
 		struct gcli_fork_list *out);
 
@@ -92,16 +97,14 @@ struct gcli_forge_descriptor {
 	 * Fork the given repo into the owner _in */
 	int (*fork_create)(
 		struct gcli_ctx *ctx,
-		char const *owner,
-		char const *repo,
+		struct gcli_path const *repo_path,
 		char const *_in);
 
 	/**
 	 * Get a list of issues on the given repo */
 	int (*search_issues)(
 		struct gcli_ctx *ctx,
-		char const *owner,
-		char const *repo,
+		struct gcli_path const *path,
 		struct gcli_issue_fetch_details const *details,
 		int max,
 		struct gcli_issue_list *out);
@@ -110,43 +113,33 @@ struct gcli_forge_descriptor {
 	 * Get a summary of an issue */
 	int (*get_issue_summary)(
 		struct gcli_ctx *ctx,
-		char const *owner,
-		char const *repo,
-		gcli_id issue_number,
+		struct gcli_path const *path,
 		struct gcli_issue *out);
 
 	/**
 	 * Close the given issue */
 	int (*issue_close)(
 		struct gcli_ctx *ctx,
-		char const *owner,
-		char const *repo,
-		gcli_id issue_number);
+		struct gcli_path const *path);
 
 	/**
 	 * Reopen the given issue */
 	int (*issue_reopen)(
 		struct gcli_ctx *ctx,
-		char const *owner,
-		char const *repo,
-		gcli_id issue_number);
+		struct gcli_path const *path);
 
 	/**
 	 * Assign an issue to a user */
 	int (*issue_assign)(
 		struct gcli_ctx *ctx,
-		char const *owner,
-		char const *repo,
-		gcli_id issue_number,
+		struct gcli_path const *issue_path,
 		char const *assignee);
 
 	/**
 	 * Add labels to issues */
 	int (*issue_add_labels)(
 		struct gcli_ctx *ctx,
-		char const *owner,
-		char const *repo,
-		gcli_id issue,
+		struct gcli_path const *issue_path,
 		char const *const labels[],
 		size_t labels_size);
 
@@ -154,9 +147,7 @@ struct gcli_forge_descriptor {
 	 * Removes labels from issues */
 	int (*issue_remove_labels)(
 		struct gcli_ctx *ctx,
-		char const *owner,
-		char const *repo,
-		gcli_id issue,
+		struct gcli_path const *issue_path,
 		char const *const labels[],
 		size_t labels_size);
 
@@ -171,20 +162,23 @@ struct gcli_forge_descriptor {
 	 * Change the title of an issue */
 	int (*issue_set_title)(
 		struct gcli_ctx *ctx,
-		char const *owner,
-		char const *repo,
-		gcli_id issue,
+		struct gcli_path const *issue_path,
 		char const *new_title);
 
 	/**
 	 * Get attachments of an issue */
 	int (*get_issue_attachments)(
 		struct gcli_ctx *ctx,
-		char const *owner,
-		char const *repo,
-		gcli_id issue,
+		struct gcli_path const *issue_path,
 		struct gcli_attachment_list *out);
 
+	/**
+	 * Change the OP (original post) of the issue */
+	 int (*issue_set_op)(
+		struct gcli_ctx *ctx,
+		struct gcli_path const *issue_path,
+		char const *new_op);
+
 	/**
 	 * Dump the contents of the attachment to the given file */
 	int (*attachment_get_content)(
@@ -198,7 +192,6 @@ struct gcli_forge_descriptor {
 		GCLI_ISSUE_QUIRKS_COMMENTS    = 0x2,
 		GCLI_ISSUE_QUIRKS_PROD_COMP   = 0x4,
 		GCLI_ISSUE_QUIRKS_URL         = 0x8,
-		GCLI_ISSUE_QUIRKS_ATTACHMENTS = 0x10,
 	} const issue_quirks;
 
 	/**
@@ -214,8 +207,7 @@ struct gcli_forge_descriptor {
 	 * Get list of milestones */
 	int (*get_milestones)(
 		struct gcli_ctx *ctx,
-		char const *owner,
-		char const *repo,
+		struct gcli_path const *path,
 		int max,
 		struct gcli_milestone_list *out);
 
@@ -223,65 +215,53 @@ struct gcli_forge_descriptor {
 	 * Get a single milestone */
 	int (*get_milestone)(
 		struct gcli_ctx *ctx,
-		char const *owner,
-		char const *repo,
-		gcli_id milestone,
+		struct gcli_path const *path,
 		struct gcli_milestone *out);
 
 	/**
 	 * create a milestone */
 	int (*create_milestone)(
 		struct gcli_ctx *ctx,
+		struct gcli_path const *repo,
 		struct gcli_milestone_create_args const *args);
 
 	/**
 	 * delete a milestone */
 	int (*delete_milestone)(
 		struct gcli_ctx *ctx,
-		char const *owner,
-		char const *repo,
-		gcli_id milestone);
+		struct gcli_path const *path);
 
 	/**
 	 * delete a milestone */
 	int (*milestone_set_duedate)(
 		struct gcli_ctx *ctx,
-		char const *owner,
-		char const *repo,
-		gcli_id milestone,
+		struct gcli_path const *path,
 		char const *date);
 
 	/**
 	 * Get list of issues attached to this milestone */
 	int (*get_milestone_issues)(
 		struct gcli_ctx *ctx,
-		char const *owner,
-		char const *repo,
-		gcli_id milestone,
+		struct gcli_path const *const path,
 		struct gcli_issue_list *out);
 
 	/** Assign an issue to a milestone */
 	int (*issue_set_milestone)(
 		struct gcli_ctx *ctx,
-		char const *owner,
-		char const *repo,
-		gcli_id issue,
+		struct gcli_path const *issue_path,
 		gcli_id milestone);
 
 	/**
 	 * Clear the milestones of an issue */
 	int (*issue_clear_milestone)(
 		struct gcli_ctx *ctx,
-		char const *owner,
-		char const *repo,
-		gcli_id issue);
+		struct gcli_path const *issue_path);
 
 	/**
 	 * Get a list of PRs/MRs on the given repo */
 	int (*search_pulls)(
 		struct gcli_ctx *ctx,
-		char const *owner,
-		char const *reponame,
+		struct gcli_path const *repo_path,
 		struct gcli_pull_fetch_details const *details,
 		int max,
 		struct gcli_pull_list *out);
@@ -291,18 +271,28 @@ struct gcli_forge_descriptor {
 	int (*pull_get_diff)(
 		struct gcli_ctx *ctx,
 		FILE *stream,
-		char const *owner,
-		char const *reponame,
-		gcli_id pr_number);
+		struct gcli_path const *path);
 
 	/**
 	 * Fetch the PR patch series into the file */
 	int (*pull_get_patch)(
 		struct gcli_ctx *ctx,
 		FILE *stream,
-		char const *owner,
-		char const *repo,
-		gcli_id pull_id);
+		struct gcli_path const *const pull_path);
+
+	/**
+	 * Fetch reviews on this pull request */
+	int (*pull_get_reviews)(
+		struct gcli_ctx *ctx,
+		struct gcli_path const *pull_path,
+		struct gcli_pull_reviews *out);
+
+	/**
+	 * get review threads */
+	int (*pull_get_review_threads)(
+		struct gcli_ctx *ctx,
+		struct gcli_path const *pull_path,
+		struct gcli_pull_review_thread *out);
 
 	/**
 	 * Return a list of checks associated with the given pull.
@@ -315,26 +305,26 @@ struct gcli_forge_descriptor {
 	 * Merge the given PR/MR */
 	int (*pull_merge)(
 		struct gcli_ctx *ctx,
-		char const *owner,
-		char const *reponame,
-		gcli_id pr_number,
+		struct gcli_path const *path,
 		enum gcli_merge_flags flags);
 
 	/**
 	 * Reopen the given PR/MR */
 	int (*pull_reopen)(
 		struct gcli_ctx *ctx,
-		char const *owner,
-		char const *reponame,
-		gcli_id pr_number);
+		struct gcli_path const *path);
 
 	/**
 	 * Close the given PR/MR */
 	int (*pull_close)(
 		struct gcli_ctx *ctx,
-		char const *owner,
-		char const *reponame,
-		gcli_id pr_number);
+		struct gcli_path const *path);
+
+	/**
+	 * Quirks when creating pull requests */
+	enum gcli_pull_quirks {
+		GCLI_PULL_QUIRK_AUTOMERGE = 0x01, /* forge does not support automerge */
+	} pull_quirks;
 
 	/**
 	 * Submit PR/MR */
@@ -346,9 +336,7 @@ struct gcli_forge_descriptor {
 	 * Get a list of commits in the given PR/MR */
 	int (*get_pull_commits)(
 		struct gcli_ctx *ctx,
-		char const *owner,
-		char const *repo,
-		gcli_id pr_number,
+		struct gcli_path const *path,
 		struct gcli_commit_list *out);
 
 	/** Bitmask of unsupported fields in the pull summary for this
@@ -367,18 +355,14 @@ struct gcli_forge_descriptor {
 	 * Get a summary of the given PR/MR */
 	int (*get_pull)(
 		struct gcli_ctx *ctx,
-		char const *owner,
-		char const *repo,
-		gcli_id pr_number,
+		struct gcli_path const *path,
 		struct gcli_pull *out);
 
 	/**
 	 * Add labels to Pull Requests */
 	int (*pull_add_labels)(
 		struct gcli_ctx *ctx,
-		char const *owner,
-		char const *repo,
-		gcli_id pr,
+		struct gcli_path const *pull_path,
 		char const *const labels[],
 		size_t labels_size);
 
@@ -386,9 +370,7 @@ struct gcli_forge_descriptor {
 	 * Removes labels from Pull Requests */
 	int (*pull_remove_labels)(
 		struct gcli_ctx *ctx,
-		char const *owner,
-		char const *repo,
-		gcli_id pr,
+		struct gcli_path const *pull_path,
 		char const *const labels[],
 		size_t labels_size);
 
@@ -396,35 +378,34 @@ struct gcli_forge_descriptor {
 	 * Assign a PR to a milestone */
 	int (*pull_set_milestone)(
 		struct gcli_ctx *ctx,
-		char const *owner,
-		char const *repo,
-		gcli_id pull,
+		struct gcli_path const *pull_path,
 		gcli_id milestone_id);
 
 	/**
 	 * Clear a milestone on a PR */
 	int (*pull_clear_milestone)(
 		struct gcli_ctx *ctx,
-		char const *owner,
-		char const *repo,
-		gcli_id pull);
+		struct gcli_path const *issue_path);
 
 	/**
-     * Request review of a given pull request by a user */
+	 * Request review of a given pull request by a user */
 	int (*pull_add_reviewer)(
 		struct gcli_ctx *ctx,
-		char const *owner,
-		char const *repo,
-		gcli_id pull,
+		struct gcli_path const *path,
+		char const *username);
+
+	/**
+	 * assign this pull request to the given user */
+	int (*pull_assign)(
+		struct gcli_ctx *ctx,
+		struct gcli_path const *path,
 		char const *username);
 
 	/**
 	 * Change the title of a pull request */
 	int (*pull_set_title)(
 		struct gcli_ctx *ctx,
-		char const *owner,
-		char const *repo,
-		gcli_id pull,
+		struct gcli_path const *path,
 		char const *new_title);
 
 	/**
@@ -433,12 +414,17 @@ struct gcli_forge_descriptor {
 		struct gcli_ctx *ctx,
 		struct gcli_pull_create_review_details const *details);
 
+	/** Checkout this PR */
+	int (*pull_checkout)(
+		struct gcli_ctx *ctx,
+		char const *remote,
+		struct gcli_path const *pull_path);
+
 	/**
 	 * Get a list of releases in the given repo */
 	int (*get_releases)(
 		struct gcli_ctx *ctx,
-		char const *owner,
-		char const *repo,
+		struct gcli_path const *repo_path,
 		int max,
 		struct gcli_release_list *out);
 
@@ -446,42 +432,65 @@ struct gcli_forge_descriptor {
 	 * Create a new release */
 	int (*create_release)(
 		struct gcli_ctx *ctx,
-		struct gcli_new_release const *release);
+		struct gcli_create_release_args const *release);
 
 	/**
 	 * Delete the release */
 	int (*delete_release)(
 		struct gcli_ctx *ctx,
-		char const *owner,
-		char const *repo,
+		struct gcli_path const *repo_path,
 		char const *id);
 
 	/**
 	 * Get a list of labels that are valid in the given repository */
-    int (*get_labels)(
-	    struct gcli_ctx *ctx,
-		char const *owner,
-		char const *repo,
+	int (*get_labels)(
+		struct gcli_ctx *ctx,
+		struct gcli_path const *path,
 		int max,
 		struct gcli_label_list *out);
 
+	/**
+	 *  Get a single label */
+	int (*get_label)(
+		struct gcli_ctx *ctx,
+		struct gcli_path const *path,
+		struct gcli_label *out);
+
 	/**
 	 * Create the given label
 	 *
 	 * The ID will be filled in for you */
 	int (*create_label)(
-	    struct gcli_ctx *ctx,
-		char const *owner,
-		char const *repo,
+		struct gcli_ctx *ctx,
+		struct gcli_path const *repo_path,
 		struct gcli_label *label);
 
 	/**
 	 * Delete the given label */
 	int (*delete_label)(
-	    struct gcli_ctx *ctx,
-		char const *owner,
-		char const *repo,
-		char const *label);
+		struct gcli_ctx *ctx,
+		struct gcli_path const *path);
+
+	/**
+	 *  change the title of a label */
+	int (*label_set_title)(
+		struct gcli_ctx *ctx,
+		struct gcli_path const *path,
+		char const *new_name);
+
+	/**
+	 *  change the description of a label */
+	int (*label_set_description)(
+		struct gcli_ctx *ctx,
+		struct gcli_path const *path,
+		char const *new_description);
+
+	/**
+	 *  change the colour of a label */
+	int (*label_set_colour)(
+		struct gcli_ctx *ctx,
+		struct gcli_path const *path,
+		uint32_t colour_rgb);
 
 	/**
 	 * Get a list of repos of the given owner */
@@ -502,15 +511,13 @@ struct gcli_forge_descriptor {
 	 * Delete the given repo */
 	int (*repo_delete)(
 		struct gcli_ctx *ctx,
-		char const *owner,
-		char const *repo);
+		struct gcli_path const *path);
 
 	/**
 	 * Change the visibility level of a repository */
 	int (*repo_set_visibility)(
 		struct gcli_ctx *ctx,
-		char const *owner,
-		char const *repo,
+		struct gcli_path const *path,
 		gcli_repo_visibility vis);
 
 	/**
@@ -520,13 +527,6 @@ struct gcli_forge_descriptor {
 		int max,
 		struct gcli_notification_list *notifications);
 
-	/**
-	 * *Given an issue notification grab the associated issue. */
-	int (*notification_get_issue)(
-		struct gcli_ctx *ctx,
-		struct gcli_notification const *const notification,
-		struct gcli_issue *out);
-
 	/**
 	 * Given a notification fetch the target's comments */
 	int (*notification_get_comments)(
