@@ -1,5 +1,5 @@
 /*
- * Copyright 2022 Nico Sonack <nsonack@herrhotzenplotz.de>
+ * Copyright 2022-2025 Nico Sonack <nsonack@herrhotzenplotz.de>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -35,8 +35,9 @@
 #endif
 
 #include <gcli/gcli.h>
+#include <gcli/path.h>
 
-#include <sn/sn.h>
+#include <stddef.h>
 
 struct gcli_github_check {
 	char *name;
@@ -52,7 +53,7 @@ struct github_check_list {
 	size_t checks_size;
 };
 
-int github_get_checks(struct gcli_ctx *ctx, char const *owner, char const *repo,
+int github_get_checks(struct gcli_ctx *ctx, struct gcli_path const *repo_path,
                       char const *ref, int max,
                       struct github_check_list *checks);
 
