@@ -1,5 +1,5 @@
 /*
- * Copyright 2022 Nico Sonack <nsonack@herrhotzenplotz.de>
+ * Copyright 2022-2025 Nico Sonack <nsonack@herrhotzenplotz.de>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -39,11 +39,13 @@
 struct gitlab_pipeline {
 	gcli_id id;
 	char *status;
-	char *created_at;
-	char *updated_at;
+	time_t created_at;
+	time_t updated_at;
 	char *ref;
 	char *sha;
 	char *source;
+	char *name;
+	char *web_url;
 };
 
 struct gitlab_pipeline_list {
@@ -57,13 +59,14 @@ struct gitlab_job {
 	char *stage;
 	char *name;
 	char *ref;
-	char *created_at;
-	char *started_at;
-	char *finished_at;
+	time_t created_at;
+	time_t started_at;
+	time_t finished_at;
 	double duration;
 	char *runner_name;
 	char *runner_description;
 	double coverage;
+	char *web_url;
 };
 
 struct gitlab_job_list {
@@ -71,38 +74,44 @@ struct gitlab_job_list {
 	size_t jobs_size;
 };
 
-int gitlab_get_pipelines(struct gcli_ctx *ctx, char const *owner,
-                         char const *repo, int max,
+int gitlab_get_pipelines(struct gcli_ctx *ctx,
+                         struct gcli_path const *repo_path, int max,
                          struct gitlab_pipeline_list *out);
 
+int gitlab_get_pipeline(struct gcli_ctx *ctx,
+                        struct gcli_path const *pipeline_path,
+                        struct gitlab_pipeline *out);
+
 void gitlab_pipeline_free(struct gitlab_pipeline *pipeline);
 void gitlab_pipelines_free(struct gitlab_pipeline_list *list);
 
-int gitlab_get_pipeline_jobs(struct gcli_ctx *ctx, char const *owner,
-                             char const *repo, gcli_id pipeline, int count,
-                             struct gitlab_job_list *out);
+int gitlab_get_pipeline_jobs(struct gcli_ctx *ctx,
+                             struct gcli_path const *pipeline_path,
+                             int count, struct gitlab_job_list *out);
+
+int gitlab_get_pipeline_children(struct gcli_ctx *ctx,
+                                 struct gcli_path const *pipeline_path,
+                                 int count, struct gitlab_pipeline_list *out);
 
 void gitlab_free_jobs(struct gitlab_job_list *jobs);
+
 void gitlab_free_job(struct gitlab_job *job);
 
-int gitlab_job_get_log(struct  gcli_ctx *ctx, char const *owner,
-                       char const *repo, gcli_id job_id, FILE *stream);
+int gitlab_job_get_log(struct  gcli_ctx *ctx, struct gcli_path const *job_path,
+                       FILE *stream);
 
-int gitlab_job_cancel(struct gcli_ctx *ctx, char const *owner, char const *repo,
-                      gcli_id job_id);
+int gitlab_job_cancel(struct gcli_ctx *ctx, struct gcli_path const *job_path);
 
-int gitlab_job_retry(struct gcli_ctx *ctx, char const *owner, char const *repo,
-                     gcli_id job_id);
+int gitlab_job_retry(struct gcli_ctx *ctx, struct gcli_path const *job_path);
 
-int gitlab_job_download_artifacts(struct gcli_ctx *ctx, char const *owner,
-                                  char const *repo, gcli_id jid,
+int gitlab_job_download_artifacts(struct gcli_ctx *ctx,
+                                  struct gcli_path const *job_path,
                                   char const *outfile);
 
-int gitlab_get_mr_pipelines(struct gcli_ctx *ctx, char const *owner,
-                            char const *repo, gcli_id mr_id,
+int gitlab_get_mr_pipelines(struct gcli_ctx *ctx, struct gcli_path const *path,
                             struct gitlab_pipeline_list *list);
 
-int gitlab_get_job(struct gcli_ctx *ctx, char const *owner, char const *repo,
-                   gcli_id const jid, struct gitlab_job *const out);
+int gitlab_get_job(struct gcli_ctx *ctx, struct gcli_path const *job_path,
+                   struct gitlab_job *const out);
 
 #endif /* GITLAB_PIPELINES_H */
