@@ -1,5 +1,5 @@
 /*
- * Copyright 2021, 2022 Nico Sonack <nsonack@herrhotzenplotz.de>
+ * Copyright 2021-2025 Nico Sonack <nsonack@herrhotzenplotz.de>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -34,17 +34,17 @@
 #include <config.h>
 #endif
 
+#include <gcli/path.h>
 #include <gcli/releases.h>
 
-int gitlab_get_releases(struct gcli_ctx *ctx, char const *owner,
-                        char const *repo, int max,
-                        struct gcli_release_list *list);
+int gitlab_get_releases(struct gcli_ctx *ctx, struct gcli_path const *repo_path,
+                        int max, struct gcli_release_list *list);
 
 int gitlab_create_release(struct gcli_ctx *ctx,
-                          struct gcli_new_release const *release);
+                          struct gcli_create_release_args const *release);
 
-int gitlab_delete_release(struct gcli_ctx *ctx, char const *owner,
-                          char const *repo, char const *id);
+int gitlab_delete_release(struct gcli_ctx *ctx, struct gcli_path const *path,
+                          char const *id);
 
 void gitlab_fixup_release_assets(struct gcli_ctx *ctx,
                                  struct gcli_release *const release);
