@@ -1,5 +1,5 @@
 /*
- * Copyright 2022 Nico Sonack <nsonack@herrhotzenplotz.de>
+ * Copyright 2022-2025 Nico Sonack <nsonack@herrhotzenplotz.de>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -39,6 +39,7 @@
 #include <stdlib.h>
 
 #include <gcli/gcli.h>
+#include <gcli/path.h>
 
 struct gcli_label {
 	gcli_id id;
@@ -52,17 +53,28 @@ struct gcli_label_list {
 	size_t labels_size;
 };
 
-int gcli_get_labels(struct gcli_ctx *ctx, char const *owner, char const *repo,
+int gcli_get_labels(struct gcli_ctx *ctx, struct gcli_path const *path,
                     int max, struct gcli_label_list *out);
 
+int gcli_get_label(struct gcli_ctx *ctx, struct gcli_path const *path,
+                   struct gcli_label *out);
+
 void gcli_free_label(struct gcli_label *label);
 
 void gcli_free_labels(struct gcli_label_list *labels);
 
-int gcli_create_label(struct gcli_ctx *ctx, char const *owner, char const *repo,
+int gcli_create_label(struct gcli_ctx *ctx, struct gcli_path const *path,
                       struct gcli_label *label);
 
-int gcli_delete_label(struct gcli_ctx *ctx, char const *owner, char const *repo,
-                      char const *label);
+int gcli_delete_label(struct gcli_ctx *ctx, struct gcli_path const *path);
+
+int gcli_label_set_title(struct gcli_ctx *ctx, struct gcli_path const *path,
+                         char const *new_name);
+
+int gcli_label_set_description(struct gcli_ctx *ctx, struct gcli_path const *path,
+                               char const *new_description);
+
+int gcli_label_set_colour(struct gcli_ctx *ctx, struct gcli_path const *path,
+                          uint32_t colour_rgb);
 
 #endif /* LABELS_H */
