@@ -1,5 +1,5 @@
 /*
- * Copyright 2021, 2022 Nico Sonack <nsonack@herrhotzenplotz.de>
+ * Copyright 2021-2025 Nico Sonack <nsonack@herrhotzenplotz.de>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -36,10 +36,11 @@
 
 #include <stdio.h>
 #include <stdbool.h>
+#include <time.h>
 
 #include <gcli/diffutil.h>
 #include <gcli/gcli.h>
-#include <sn/sn.h>
+#include <gcli/path.h>
 
 struct gcli_pull_list {
 	struct gcli_pull *pulls;
@@ -51,7 +52,7 @@ struct gcli_pull {
 	char *state;
 	char *title;
 	char *body;
-	char *created_at;
+	time_t created_at;
 	char *commits_link;
 	char *head_label;
 	char *base_label;
@@ -77,6 +78,9 @@ struct gcli_pull {
 	char **reviewers;      /**< User names */
 	size_t reviewers_size; /**< Number of elements in the reviewers list */
 
+	char **assignees;
+	size_t assignees_size;
+
 	bool merged;
 	bool mergeable;
 	bool draft;
@@ -94,14 +98,15 @@ struct gcli_commit_list {
 
 /* Options to submit to the gh api for creating a PR */
 struct gcli_submit_pull_options {
-	char const *owner;
-	char const *repo;
+	struct gcli_path target_repo;
+	char const *target_branch;
 	char const *from;
-	char const *to;
 	char const *title;
 	char *body;
 	char **labels;
 	size_t labels_size;
+	char **reviewers;
+	size_t reviewers_size;
 	int draft;
 	bool automerge;           /** Automatically merge the PR when a pipeline passes */
 };
@@ -120,16 +125,20 @@ enum {
 	GCLI_REVIEW_COMMENT = 3,
 };
 
+enum gcli_pull_approval_state {
+	GCLI_PULL_APPROVED = GCLI_REVIEW_ACCEPT_CHANGES,
+	GCLI_PULL_UNAPPROVED = GCLI_REVIEW_REQUEST_CHANGES,
+};
+
 struct gcli_review_meta_line {
 	TAILQ_ENTRY(gcli_review_meta_line) next;
 	char *entry;
 };
 
 struct gcli_pull_create_review_details {
-	char const *owner, *repo;
-	gcli_id pull_id;
+	struct gcli_path path;
 	struct gcli_diff_comments comments;
-	char *body;       /* string containing the prelude message by the user */
+	char const *body;       /* string containing the prelude message by the user */
 	TAILQ_HEAD(, gcli_review_meta_line) meta_lines;
 	int review_state;
 };
@@ -148,7 +157,45 @@ struct gcli_pull_checks_list {
 	int forge_type;
 };
 
-int gcli_search_pulls(struct gcli_ctx *ctx, char const *owner, char const *repo,
+/* PR/MR reviews */
+struct gcli_pull_review {
+	gcli_id id;
+	char *author;
+	char *state;
+	time_t submitted_at;
+	char *body;
+};
+
+struct gcli_pull_reviews {
+	struct gcli_pull_review *reviews;
+	size_t reviews_size;
+};
+
+/* comments in a review */
+struct gcli_pull_review_comment;
+TAILQ_HEAD(gcli_pull_review_thread, gcli_pull_review_comment);
+
+struct gcli_pull_review_comment {
+	/* threaded comments */
+	TAILQ_ENTRY(gcli_pull_review_comment) next;
+	struct gcli_pull_review_thread replies;
+
+	/* actual fields */
+	gcli_id id;
+	gcli_id in_reply_to;
+	char *author;
+	char *body;
+	char *path;
+	char *diff_hunk;
+	time_t created_at;
+};
+
+struct gcli_pull_review_comments {
+	struct gcli_pull_review_comment *comments;
+	size_t comments_size;
+};
+
+int gcli_search_pulls(struct gcli_ctx *ctx, struct gcli_path const *path,
                       struct gcli_pull_fetch_details const *details, int max,
                       struct gcli_pull_list *out);
 
@@ -156,23 +203,22 @@ void gcli_pull_free(struct gcli_pull *it);
 
 void gcli_pulls_free(struct gcli_pull_list *list);
 
-int gcli_pull_get_diff(struct gcli_ctx *ctx, FILE *fout, char const *owner,
-                       char const *repo, gcli_id pr_number);
+int gcli_pull_get_diff(struct gcli_ctx *ctx, FILE *fout,
+                       struct gcli_path const *path);
 
-int gcli_pull_get_checks(struct gcli_ctx *ctx, char const *owner,
-                         char const *repo, gcli_id pr_number,
+int gcli_pull_get_checks(struct gcli_ctx *ctx,
+                         struct gcli_path const *const path,
                          struct gcli_pull_checks_list *out);
 
 void gcli_pull_checks_free(struct gcli_pull_checks_list *list);
 
-int gcli_pull_get_commits(struct gcli_ctx *ctx, char const *owner,
-                          char const *repo, gcli_id pr_number,
+int gcli_pull_get_commits(struct gcli_ctx *ctx, struct gcli_path const *path,
                           struct gcli_commit_list *out);
 
 void gcli_commits_free(struct gcli_commit_list *list);
 
-int gcli_get_pull(struct gcli_ctx *ctx, char const *owner, char const *repo,
-                  gcli_id pr_number, struct gcli_pull *out);
+int gcli_get_pull(struct gcli_ctx *ctx, struct gcli_path const *path,
+                  struct gcli_pull *out);
 
 int gcli_pull_submit(struct gcli_ctx *ctx, struct gcli_submit_pull_options *);
 
@@ -181,47 +227,64 @@ enum gcli_merge_flags {
 	GCLI_PULL_MERGE_DELETEHEAD = 0x2, /* delete the source branch after merging */
 };
 
-int gcli_pull_merge(struct gcli_ctx *ctx, char const *owner, char const *repo,
-                    gcli_id pr_number, enum gcli_merge_flags flags);
+int gcli_pull_merge(struct gcli_ctx *ctx, struct gcli_path const *const path,
+                    enum gcli_merge_flags flags);
 
-int gcli_pull_close(struct gcli_ctx *ctx, char const *owner, char const *repo,
-                    gcli_id pr_number);
+int gcli_pull_close(struct gcli_ctx *ctx, struct gcli_path const *const path);
 
-int gcli_pull_reopen(struct gcli_ctx *ctx, char const *owner, char const *repo,
-                     gcli_id pr_number);
+int gcli_pull_reopen(struct gcli_ctx *ctx, struct gcli_path const *const path);
 
-int gcli_pull_add_labels(struct gcli_ctx *ctx, char const *owner,
-                         char const *repo, gcli_id pr_number,
+int gcli_pull_add_labels(struct gcli_ctx *ctx, struct gcli_path const *path,
                          char const *const labels[], size_t labels_size);
 
-int gcli_pull_remove_labels(struct gcli_ctx *ctx, char const *owner,
-                            char const *repo, gcli_id pr_number,
+int gcli_pull_remove_labels(struct gcli_ctx *ctx, struct gcli_path const *path,
                             char const *const labels[], size_t labels_size);
 
-int gcli_pull_set_milestone(struct gcli_ctx *ctx, char const *owner,
-                            char const *repo, gcli_id pr_number,
+int gcli_pull_set_milestone(struct gcli_ctx *ctx,
+                            struct gcli_path const *pull_path,
                             int milestone_id);
 
-int gcli_pull_clear_milestone(struct gcli_ctx *ctx, char const *owner,
-                              char const *repo, gcli_id pr_number);
+int gcli_pull_clear_milestone(struct gcli_ctx *ctx,
+                              struct gcli_path const *pull_path);
 
-int gcli_pull_add_reviewer(struct gcli_ctx *ctx, char const *owner,
-                           char const *repo, gcli_id pr_number,
+int gcli_pull_add_reviewer(struct gcli_ctx *ctx, struct gcli_path const *path,
                            char const *username);
 
-int gcli_pull_get_patch(struct gcli_ctx *ctx, FILE *out, char const *owner,
-                        char const *repo, gcli_id pr_number);
+int gcli_pull_assign(struct gcli_ctx *ctx, struct gcli_path const *path,
+                     char const *username);
 
-int gcli_pull_set_title(struct gcli_ctx *ctx, char const *owner,
-                        char const *repo, gcli_id pull, char const *new_title);
+int gcli_pull_get_patch(struct gcli_ctx *ctx, FILE *out,
+                        struct gcli_path const *path);
+
+int gcli_pull_set_title(struct gcli_ctx *ctx, struct gcli_path const *path,
+                        char const *new_title);
 
 int gcli_pull_create_review(struct gcli_ctx *ctx,
                             struct gcli_pull_create_review_details const *details);
 
-int gcli_pull_get_patch(struct gcli_ctx *ctx, FILE *out, char const *owner,
-                        char const *repo, gcli_id pr_number);
-
 char const *gcli_pull_get_meta_by_key(struct gcli_pull_create_review_details const *,
                                       char const *key);
 
+int gcli_pull_checkout(struct gcli_ctx *ctx, char const *remote,
+                       struct gcli_path const *pull_path);
+
+int gcli_pull_get_reviews(struct gcli_ctx *ctx, struct gcli_path const *path,
+                          struct gcli_pull_reviews *out);
+
+void gcli_pull_reviews_free(struct gcli_pull_reviews *it);
+
+void gcli_pull_review_comments_free(struct gcli_pull_review_comments *it);
+
+int gcli_pull_get_review_threads(struct gcli_ctx *ctx,
+                                 struct gcli_path const *path,
+                                 struct gcli_pull_review_thread *out);
+
+void gcli_pull_review_thread_free(struct gcli_pull_review_thread *thd);
+
+int gcli_pull_approve(struct gcli_ctx *ctx, struct gcli_path const *path,
+                      char const *const message);
+
+int gcli_pull_unapprove(struct gcli_ctx *ctx, struct gcli_path const *path,
+                        char const *const message);
+
 #endif /* PULLS_H */
