@@ -1,5 +1,5 @@
 /*
- * Copyright 2021, 2022 Nico Sonack <nsonack@herrhotzenplotz.de>
+ * Copyright 2021-2025 Nico Sonack <nsonack@herrhotzenplotz.de>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -34,15 +34,17 @@
 #include <config.h>
 #endif
 
-#include <sn/sn.h>
 #include <gcli/gcli.h>
+#include <gcli/path.h>
+
+#include <time.h>
 
 struct gcli_repo {
 	gcli_id id;
 	char *full_name;
 	char *name;
 	char *owner;
-	char *date;
+	time_t date;
 	char *visibility;
 	bool is_fork;
 };
@@ -69,13 +71,14 @@ int gcli_get_repos(struct gcli_ctx *ctx, char const *owner, int max,
 void gcli_repos_free(struct gcli_repo_list *list);
 void gcli_repo_free(struct gcli_repo *it);
 
-int gcli_repo_delete(struct gcli_ctx *ctx, char const *owner, char const *repo);
+int gcli_repo_delete(struct gcli_ctx *ctx, struct gcli_path const *path);
 
 int gcli_repo_create(struct gcli_ctx *ctx,
                      struct gcli_repo_create_options const *,
                      struct gcli_repo *out);
 
-int gcli_repo_set_visibility(struct gcli_ctx *ctx, char const *owner,
-                             char const *repo, gcli_repo_visibility visibility);
+int gcli_repo_set_visibility(struct gcli_ctx *ctx,
+                             struct gcli_path const *const path,
+                             gcli_repo_visibility visibility);
 
 #endif /* REPOS_H */
