@@ -1,5 +1,5 @@
 /*
- * Copyright 2021, 2022 Nico Sonack <nsonack@herrhotzenplotz.de>
+ * Copyright 2021-2025 Nico Sonack <nsonack@herrhotzenplotz.de>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -34,26 +34,27 @@
 #include <gcli/cmd/gitconfig.h>
 
 #include <gcli/ctx.h>
+#include <gcli/gcli.h>
 #include <gcli/forges.h>
 #include <gcli/gitea/config.h>
 #include <gcli/github/config.h>
 #include <gcli/gitlab/config.h>
+#include <gcli/port/string.h>
+#include <gcli/port/util.h>
 
 #include <assert.h>
 #include <ctype.h>
 #include <dirent.h>
-#ifdef HAVE_GETOPT_H
 #include <getopt.h>
-#endif
 #include <stdlib.h>
 #include <unistd.h>
 
 struct gcli_config_section {
 	TAILQ_ENTRY(gcli_config_section) next;
 
-    struct gcli_config_entries entries;
+	struct gcli_config_entries entries;
 
-	sn_sv title;
+	gcli_sv title;
 };
 
 struct gcli_config {
@@ -65,20 +66,21 @@ struct gcli_config {
 	int colours_disabled;       /* NO_COLOR set or output is not a TTY */
 	int force_colours;          /* -c option was given */
 	int no_spinner;             /* don't show a progress spinner */
+	int no_markdown;            /* do not render markdown (when built with lowdown) */
 	int enable_experimental;    /* enable experimental features */
 
-	sn_sv  buffer;
-	void  *mmap_pointer;
-	bool   inited;
+	gcli_sv buffer;
+	char    *file_content;
+	bool    inited;
 };
 
 struct gcli_dotgcli {
 	struct gcli_config_entries entries;
 
-	sn_sv  buffer;
-	void  *mmap_pointer;
-	bool   has_been_searched_for;
-	bool   has_been_found;
+	gcli_sv buffer;
+	char    *file_content;
+	bool    has_been_searched_for;
+	bool    has_been_found;
 };
 
 struct cmd_ctx {
@@ -156,7 +158,7 @@ find_dotgcli(void)
 
 			free(curr_dir_path);
 
-			curr_dir_path = realpath(tmp, NULL);
+			curr_dir_path = gcli_cmd_realpath(tmp);
 			if (!curr_dir_path)
 				err(1, "gcli: realpath at %s", tmp);
 
@@ -200,18 +202,18 @@ init_local_config(struct gcli_ctx *ctx)
 	dgcli->has_been_searched_for = true;
 	dgcli->has_been_found = true;
 
-	int len = sn_mmap_file(path, &dgcli->mmap_pointer);
+	int len = gcli_read_file(path, &dgcli->file_content);
 	if (len < 0)
 		err(1, "gcli: unable to open config file");
 
-	dgcli->buffer = sn_sv_from_parts(dgcli->mmap_pointer, len);
-	dgcli->buffer = sn_sv_trim_front(dgcli->buffer);
+	dgcli->buffer = gcli_sv_from_parts(dgcli->file_content, len);
+	dgcli->buffer = gcli_sv_trim_front(dgcli->buffer);
 
 	int curr_line = 1;
 	while (dgcli->buffer.length > 0) {
-		sn_sv line = sn_sv_chop_until(&dgcli->buffer, '\n');
+		gcli_sv line = gcli_sv_chop_until(&dgcli->buffer, '\n');
 
-		line = sn_sv_trim(line);
+		line = gcli_sv_trim(line);
 
 		if (line.length == 0)
 			errx(1, "gcli: %s:%d: Unexpected end of line",
@@ -219,14 +221,14 @@ init_local_config(struct gcli_ctx *ctx)
 
 		// Comments
 		if (line.data[0] == '#') {
-			dgcli->buffer = sn_sv_trim_front(dgcli->buffer);
+			dgcli->buffer = gcli_sv_trim_front(dgcli->buffer);
 			curr_line++;
 			continue;
 		}
 
-		sn_sv key = sn_sv_chop_until(&line, '=');
+		gcli_sv key = gcli_sv_chop_until(&line, '=');
 
-		key = sn_sv_trim(key);
+		key = gcli_sv_trim(key);
 
 		if (key.length == 0)
 			errx(1, "gcli: %s:%d: empty key", path, curr_line);
@@ -234,15 +236,15 @@ init_local_config(struct gcli_ctx *ctx)
 		line.data   += 1;
 		line.length -= 1;
 
-		sn_sv value = sn_sv_trim(line);
+		gcli_sv value = gcli_sv_trim(line);
 
-		struct gcli_config_entry *entry = calloc(sizeof(*entry), 1);
+		struct gcli_config_entry *entry = calloc(1, sizeof(*entry));
 		TAILQ_INSERT_TAIL(&dgcli->entries, entry, next);
 
 	    entry->key = key;
 	    entry->value = value;
 
-		dgcli->buffer = sn_sv_trim_front(dgcli->buffer);
+		dgcli->buffer = gcli_sv_trim_front(dgcli->buffer);
 		curr_line++;
 	}
 
@@ -250,7 +252,7 @@ init_local_config(struct gcli_ctx *ctx)
 }
 
 struct config_parser {
-	sn_sv buffer;
+	gcli_sv buffer;
 	int line;
 	char const *filename;
 };
@@ -280,7 +282,7 @@ again:
 not_whitespace:
 	if (input->buffer.data[0] == '#') {
 		/* This is a comment */
-		sn_sv_chop_until(&input->buffer, '\n');
+		gcli_sv_chop_until(&input->buffer, '\n');
 		goto again;
 	}
 }
@@ -289,10 +291,10 @@ static void
 parse_section_entry(struct config_parser *input,
                     struct gcli_config_section *section)
 {
-	struct gcli_config_entry *entry = calloc(sizeof(*entry), 1);
+	struct gcli_config_entry *entry = calloc(1, sizeof(*entry));
 	TAILQ_INSERT_TAIL(&section->entries, entry, next);
 
-	sn_sv key = sn_sv_chop_until(&input->buffer, '=');
+	gcli_sv key = gcli_sv_chop_until(&input->buffer, '=');
 
 	if (key.length == 0)
 		errx(1, "gcli: %s:%d: empty key", input->filename, input->line);
@@ -300,13 +302,13 @@ parse_section_entry(struct config_parser *input,
 	input->buffer.data   += 1;
 	input->buffer.length -= 1;
 
-	sn_sv value = sn_sv_chop_until(&input->buffer, '\n');
+	gcli_sv value = gcli_sv_chop_until(&input->buffer, '\n');
 
-	entry->key   = sn_sv_trim(key);
-	entry->value = sn_sv_trim(value);
+	entry->key   = gcli_sv_trim(key);
+	entry->value = gcli_sv_trim(value);
 }
 
-static sn_sv
+static gcli_sv
 parse_section_title(struct config_parser *input)
 {
 	size_t len = 0;
@@ -318,7 +320,7 @@ parse_section_title(struct config_parser *input)
 	while (!isspace(input->buffer.data[len]) && input->buffer.data[len] != '{')
 		len++;
 
-	sn_sv title = sn_sv_from_parts(input->buffer.data, len);
+	gcli_sv title = gcli_sv_from_parts(input->buffer.data, len);
 	input->buffer.data   += len;
 	input->buffer.length -= len;
 
@@ -343,7 +345,7 @@ parse_config_section(struct gcli_config *cfg,
 {
 	struct gcli_config_section *section = NULL;
 
-	section = calloc(sizeof(*section), 1);
+	section = calloc(1, sizeof(*section));
 	TAILQ_INSERT_TAIL(&cfg->sections, section, next);
 
 	section->title = parse_section_title(input);
@@ -397,28 +399,28 @@ ensure_config(struct gcli_ctx *ctx)
 	if (!file_path) {
 		file_path = getenv("HOME");
 		if (!file_path) {
-			warnx("Neither XDG_CONFIG_HOME nor HOME set in env");
+			gcli_warnx(ctx, "Neither XDG_CONFIG_HOME nor HOME set in env");
 			return cfg;
 		}
 
 		/*
 		 * Code duplication to avoid leaking pointers */
-		file_path = sn_asprintf("%s/.config/gcli/config", file_path);
+		file_path = gcli_asprintf("%s/.config/gcli/config", file_path);
 	} else {
-		file_path = sn_asprintf("%s/gcli/config", file_path);
+		file_path = gcli_asprintf("%s/gcli/config", file_path);
 	}
 
 	if (access(file_path, R_OK) < 0) {
-		warn("gcli: cannot access config file at %s", file_path);
+		gcli_warn(ctx, "gcli: cannot access config file at %s", file_path);
 		return cfg;
 	}
 
-	int len = sn_mmap_file(file_path, &cfg->mmap_pointer);
+	int len = gcli_read_file(file_path, &cfg->file_content);
 	if (len < 0)
 		err(1, "gcli: unable to open config file");
 
-	cfg->buffer = sn_sv_from_parts(cfg->mmap_pointer, len);
-	cfg->buffer = sn_sv_trim_front(cfg->buffer);
+	cfg->buffer = gcli_sv_from_parts(cfg->file_content, len);
+	cfg->buffer = gcli_sv_trim_front(cfg->buffer);
 
 	parser.buffer   = cfg->buffer;
 	parser.line     = 1;
@@ -433,22 +435,29 @@ ensure_config(struct gcli_ctx *ctx)
 
 /** Check input for a value that indicates yes/true */
 static int
-checkyes(char const *const tmp)
+string_means_true(char const *const tmp)
 {
-	static char const *const yeses[] = { "1", "y", "Y" };
+	size_t tmplen = strlen(tmp) + 1;
+	char *tmp_lower = malloc(tmplen);
 
-	if (strlen(tmp) == 3) {
-		if (tolower(tmp[0]) == 'y' && tolower(tmp[1]) == 'e' &&
-		    tolower(tmp[2]) == 's')
-			return 1;
+	strncpy(tmp_lower, tmp, tmplen);
+
+	for (size_t i = 0; i < tmplen - 1; ++i) {
+		tmp_lower[i] = tolower(tmp_lower[i]);
 	}
 
-	for (size_t i = 0; i < ARRAY_SIZE(yeses); ++i) {
-		if (strcmp(yeses[i], tmp) == 0)
-			return 1;
-	}
+	int is_yes = strcmp(tmp_lower, "1") == 0 ||
+		strcmp(tmp_lower, "yes") == 0 ||
+		strcmp(tmp_lower, "true") == 0;
 
-	return 0;
+	free(tmp_lower);
+	return is_yes;
+}
+
+static bool
+string_means_false(char const *const tmp)
+{
+	return !string_means_true(tmp);
 }
 
 /* readenv: Read values of environment variables and pre-populate the
@@ -473,19 +482,22 @@ readenv(struct gcli_config *cfg)
 	 * violate the definition to get expected and sane behaviour. */
 	tmp = getenv("NO_COLOR");
 	if (tmp && tmp[0] != '\0')
-		cfg->colours_disabled = checkyes(tmp);
+		cfg->colours_disabled = string_means_true(tmp);
 
 	if ((tmp = getenv("GCLI_NOSPINNER")))
-		cfg->no_spinner = checkyes(tmp);
+		cfg->no_spinner = string_means_true(tmp);
+
+	if ((tmp = getenv("GCLI_RENDER_MARKDOWN")))
+		cfg->no_markdown = string_means_false(tmp);
 
 	if ((tmp = getenv("GCLI_ENABLE_EXPERIMENTAL")))
-		cfg->enable_experimental = checkyes(tmp);
+		cfg->enable_experimental = string_means_true(tmp);
 }
 
 int
 gcli_config_init_ctx(struct gcli_ctx *ctx)
 {
-	struct cmd_ctx *cctx = calloc(sizeof(*cctx), 1);
+	struct cmd_ctx *cctx = calloc(1, sizeof(*cctx));
 	gcli_set_userdata(ctx, cctx);
 
 	cctx->config.sections =
@@ -527,6 +539,10 @@ gcli_config_parse_args(struct gcli_ctx *ctx, int *argc, char ***argv)
 		  .has_arg = no_argument,
 		  .flag    = &cfg->no_spinner,
 		  .val     = 1 },
+		{ .name    = "no-markdown",
+		  .has_arg = no_argument,
+		  .flag    = &cfg->no_markdown,
+		  .val     = 1 },
 		{ .name    = "type",
 		  .has_arg = required_argument,
 		  .flag    = NULL,
@@ -547,7 +563,7 @@ gcli_config_parse_args(struct gcli_ctx *ctx, int *argc, char ***argv)
 	};
 
 	/* by default we are not verbose */
-	sn_setverbosity(VERBOSITY_NORMAL);
+	gcli_setverbosity(g_clictx, GCLI_VERBOSITY_NORMAL);
 
 	/* Before we parse options, invalidate the override type so it
 	 * doesn't get confused later */
@@ -568,10 +584,10 @@ gcli_config_parse_args(struct gcli_ctx *ctx, int *argc, char ***argv)
 			cfg->force_colours = 1;
 		} break;
 		case 'q': {
-			sn_setverbosity(VERBOSITY_QUIET);
+			gcli_setverbosity(ctx, GCLI_VERBOSITY_QUIET);
 		} break;
 		case 'v': {
-			sn_setverbosity(VERBOSITY_VERBOSE);
+			gcli_setverbosity(ctx, GCLI_VERBOSITY_VERBOSE);
 		} break;
 		case 't': {
 			if (strcmp(optarg, "github") == 0) {
@@ -624,7 +640,7 @@ find_section(struct gcli_config *cfg, char const *name)
 	struct gcli_config_section *section;
 
 	TAILQ_FOREACH(section, &cfg->sections, next) {
-		if (sn_sv_eq_to(section->title, name))
+		if (gcli_sv_eq_to(section->title, name))
 			return section;
 	}
 	return NULL;
@@ -645,7 +661,7 @@ gcli_config_get_section_entries(struct gcli_ctx *ctx, char const *section_name)
 		return &s->entries;
 }
 
-sn_sv
+gcli_sv
 gcli_config_find_by_key(struct gcli_ctx *ctx, char const *section_name,
                         char const *key)
 {
@@ -656,26 +672,26 @@ gcli_config_find_by_key(struct gcli_ctx *ctx, char const *section_name,
 		find_section(cfg, section_name);
 
 	if (!section) {
-		warnx("gcli: no config section with name '%s'", section_name);
+		gcli_warnx(ctx, "gcli: no config section with name '%s'", section_name);
 		return SV_NULL;
 	}
 
 	TAILQ_FOREACH(entry, &section->entries, next) {
-		if (sn_sv_eq_to(entry->key, key))
+		if (gcli_sv_eq_to(entry->key, key))
 			return entry->value;
 	}
 
 	return SV_NULL;
 }
 
-static sn_sv
+static gcli_sv
 gcli_local_config_find_by_key(struct gcli_ctx *ctx, char const *const key)
 {
 	struct gcli_dotgcli *lcfg = ctx_dotgcli(ctx);
 	struct gcli_config_entry *entry;
 
 	TAILQ_FOREACH(entry, &lcfg->entries, next) {
-		if (sn_sv_eq_to(entry->key, key))
+		if (gcli_sv_eq_to(entry->key, key))
 			return entry->value;
 	}
 
@@ -687,7 +703,7 @@ gcli_config_get_editor(struct gcli_ctx *ctx)
 {
 	ensure_config(ctx);
 
-	return sn_sv_to_cstr(gcli_config_find_by_key(ctx, "defaults", "editor"));
+	return gcli_sv_to_cstr(gcli_config_find_by_key(ctx, "defaults", "editor"));
 }
 
 char *
@@ -695,7 +711,17 @@ gcli_config_get_pager(struct gcli_ctx *ctx)
 {
 	ensure_config(ctx);
 
-	return sn_sv_to_cstr(gcli_config_find_by_key(ctx, "defaults", "pager"));
+	return gcli_sv_to_cstr(gcli_config_find_by_key(ctx, "defaults", "pager"));
+}
+
+char *
+gcli_config_get_url_open_program(struct gcli_ctx *ctx)
+{
+	ensure_config(ctx);
+
+	return gcli_sv_to_cstr(
+		gcli_config_find_by_key(ctx, "defaults", "url-open-program")
+	);
 }
 
 static char const *const
@@ -709,12 +735,12 @@ static char *
 get_default_account(struct gcli_ctx *ctx, gcli_forge_type ftype)
 {
 	char const *const defaultname = default_account_entry_names[ftype];
-	sn_sv act = gcli_config_find_by_key(ctx, "defaults", defaultname);
+	gcli_sv act = gcli_config_find_by_key(ctx, "defaults", defaultname);
 
 	if (!act.length)
 		return NULL;
 
-	return sn_sv_to_cstr(act);
+	return gcli_sv_to_cstr(act);
 }
 
 static char *
@@ -747,9 +773,20 @@ gcli_config_get_apibase(struct gcli_ctx *ctx)
 	char *url = NULL;
 
 	if (acct) {
-		sn_sv url_sv = gcli_config_find_by_key(ctx, acct, "apibase");
-		if (url_sv.length)
-			url = sn_sv_to_cstr(url_sv);
+		gcli_sv url_sv = {0};
+
+		url_sv = gcli_config_find_by_key(ctx, acct, "api-base");
+
+		/* https://github.com/herrhotzenplotz/gcli/issues/138
+		 *
+		 * Above is correct behaviour. Below is bad/buggy behaviour.
+		 * Check for the second (buggy) option apibase and
+		 * use it if needed. */
+		if (gcli_sv_null(url_sv))
+			url_sv = gcli_config_find_by_key(ctx, acct, "apibase");
+
+		if (!gcli_sv_null(url_sv))
+			url = gcli_sv_to_cstr(url_sv);
 	}
 
 	if (!url)
@@ -765,19 +802,22 @@ char *
 gcli_config_get_account_name(struct gcli_ctx *ctx)
 {
 	char *account = gcli_config_get_account(ctx);
-	sn_sv actname = gcli_config_find_by_key(
-		ctx, account, "account");
+	gcli_sv actname;
 
+	if (!account)
+		return NULL;
+
+	actname = gcli_config_find_by_key(ctx, account, "account");
 	free(account);
 
-	return sn_sv_to_cstr(actname);
+	return gcli_sv_to_cstr(actname);
 }
 
 static char *
 get_account_token(struct gcli_ctx *ctx)
 {
 	char *account;
-	sn_sv token;
+	gcli_sv token;
 
 	account = gcli_config_get_account(ctx);
 	if (!account)
@@ -787,7 +827,7 @@ get_account_token(struct gcli_ctx *ctx)
 
 	free(account);
 
-	return sn_sv_to_cstr(token);
+	return gcli_sv_to_cstr(token);
 }
 
 char *
@@ -798,7 +838,7 @@ gcli_config_get_token(struct gcli_ctx *ctx)
 	return get_account_token(ctx);
 }
 
-sn_sv
+gcli_sv
 gcli_config_get_upstream(struct  gcli_ctx *ctx)
 {
 	init_local_config(ctx);
@@ -809,23 +849,23 @@ gcli_config_get_upstream(struct  gcli_ctx *ctx)
 bool
 gcli_config_pr_inhibit_delete_source_branch(struct gcli_ctx *ctx)
 {
-	sn_sv val;
+	gcli_sv val;
 
 	init_local_config(ctx);
 
 	val = gcli_local_config_find_by_key(ctx, "pr.inhibit-delete-source-branch");
 
-	return sn_sv_eq_to(val,	"yes");
+	return gcli_sv_eq_to(val,	"yes");
 }
 
 void
-gcli_config_get_upstream_parts(struct gcli_ctx *ctx, sn_sv *const owner,
-                               sn_sv *const repo)
+gcli_config_get_upstream_parts(struct gcli_ctx *ctx, gcli_sv *const owner,
+                               gcli_sv *const repo)
 {
 	ensure_config(ctx);
 
-	sn_sv upstream = gcli_config_get_upstream(ctx);
-	*owner = sn_sv_chop_until(&upstream, '/');
+	gcli_sv upstream = gcli_config_get_upstream(ctx);
+	*owner = gcli_sv_chop_until(&upstream, '/');
 
 	/* Sanity check: did we actually reach the '/'? */
 	if (*upstream.data != '/')
@@ -836,7 +876,7 @@ gcli_config_get_upstream_parts(struct gcli_ctx *ctx, sn_sv *const owner,
 	*repo            = upstream;
 }
 
-sn_sv
+gcli_sv
 gcli_config_get_base(struct gcli_ctx *ctx)
 {
 	init_local_config(ctx);
@@ -844,7 +884,7 @@ gcli_config_get_base(struct gcli_ctx *ctx)
 	return gcli_local_config_find_by_key(ctx, "pr.base");
 }
 
-sn_sv
+gcli_sv
 gcli_config_get_override_default_account(struct gcli_ctx *ctx)
 {
 	struct gcli_config *cfg;
@@ -870,12 +910,12 @@ gcli_config_get_forge_type_internal(struct gcli_ctx *ctx)
 	ensure_config(ctx);
 	init_local_config(ctx);
 
-	sn_sv entry = {0};
+	gcli_sv entry = {0};
 
 	if (cfg->override_default_account) {
 	    char const *section = cfg->override_default_account;
 		entry = gcli_config_find_by_key(ctx, section, "forge-type");
-		if (sn_sv_null(entry))
+		if (gcli_sv_null(entry))
 			errx(1,
 			     "gcli: error: given default override account not found or "
 			     "missing forge-type");
@@ -883,14 +923,14 @@ gcli_config_get_forge_type_internal(struct gcli_ctx *ctx)
 		entry = gcli_local_config_find_by_key(ctx, "forge-type");
 	}
 
-	if (!sn_sv_null(entry)) {
-		if (sn_sv_eq_to(entry, "github"))
+	if (!gcli_sv_null(entry)) {
+		if (gcli_sv_eq_to(entry, "github"))
 			return GCLI_FORGE_GITHUB;
-		else if (sn_sv_eq_to(entry, "gitlab"))
+		else if (gcli_sv_eq_to(entry, "gitlab"))
 			return GCLI_FORGE_GITLAB;
-		else if (sn_sv_eq_to(entry, "gitea"))
+		else if (gcli_sv_eq_to(entry, "gitea"))
 			return GCLI_FORGE_GITEA;
-		else if (sn_sv_eq_to(entry, "bugzilla"))
+		else if (gcli_sv_eq_to(entry, "bugzilla"))
 			return GCLI_FORGE_BUGZILLA;
 		else
 			errx(1, "gcli: unknown forge type "SV_FMT, SV_ARGS(entry));
@@ -911,7 +951,7 @@ gcli_config_get_forge_type(struct gcli_ctx *ctx)
 	gcli_forge_type const result = gcli_config_get_forge_type_internal(ctx);
 
 	/* print the type if verbose */
-	if (sn_verbose()) {
+	if (gcli_be_verbose(ctx)) {
 		static int have_printed_forge_type = 0;
 		static char const *const ftype_name[] = {
 			[GCLI_FORGE_GITHUB] = "GitHub",
@@ -929,11 +969,31 @@ gcli_config_get_forge_type(struct gcli_ctx *ctx)
 	return result;
 }
 
+int
+gcli_config_get_remote(struct gcli_ctx *ctx, char const **remote)
+{
+	struct gcli_config *cfg;
+	gcli_forge_type type;
+	int rc;
+
+	cfg = ensure_config(ctx);
+
+	if (cfg->override_remote) {
+		*remote = strdup(cfg->override_remote);
+		return 0;
+	}
+
+	type = gcli_config_get_forge_type(ctx);
+	rc = gcli_gitconfig_get_remote(ctx, type, remote);
+
+	return rc;
+}
+
 int
 gcli_config_get_repo(struct gcli_ctx *ctx, char const **const owner,
                      char const **const repo)
 {
-	sn_sv upstream = {0};
+	gcli_sv upstream = {0};
 	struct gcli_config *cfg;
 
 	cfg = ensure_config(ctx);
@@ -956,11 +1016,11 @@ gcli_config_get_repo(struct gcli_ctx *ctx, char const **const owner,
 	}
 
 	if ((upstream = gcli_config_get_upstream(ctx)).length != 0) {
-		sn_sv const owner_sv = sn_sv_chop_until(&upstream, '/');
-		sn_sv const repo_sv = sn_sv_from_parts(upstream.data + 1, upstream.length - 1);
+		gcli_sv const owner_sv = gcli_sv_chop_until(&upstream, '/');
+		gcli_sv const repo_sv = gcli_sv_from_parts(upstream.data + 1, upstream.length - 1);
 
-		*owner = sn_sv_to_cstr(owner_sv);
-		*repo  = sn_sv_to_cstr(repo_sv);
+		*owner = gcli_sv_to_cstr(owner_sv);
+		*repo  = gcli_sv_to_cstr(repo_sv);
 
 		return 0;
 	}
@@ -968,19 +1028,19 @@ gcli_config_get_repo(struct gcli_ctx *ctx, char const **const owner,
 	return gcli_gitconfig_repo_by_remote(ctx, NULL, owner, repo, NULL);
 }
 
-int
+bool
 gcli_config_have_colours(struct gcli_ctx *ctx)
 {
-	static int tested_tty = 0;
+	static bool tested_tty = 0;
 	struct gcli_config *cfg;
 
 	cfg = ctx_config(ctx);
 
 	if (cfg->force_colours)
-		return 1;
+		return true;
 
 	if (cfg->colours_disabled)
-		return 0;
+		return false;
 
 	if (tested_tty)
 		return !cfg->colours_disabled;
@@ -990,12 +1050,12 @@ gcli_config_have_colours(struct gcli_ctx *ctx)
 	else
 		cfg->colours_disabled = true;
 
-	tested_tty = 1;
+	tested_tty = true;
 
 	return !cfg->colours_disabled;
 }
 
-int
+bool
 gcli_config_display_progress_spinner(struct gcli_ctx *ctx)
 {
 	ensure_config(ctx);
@@ -1004,16 +1064,39 @@ gcli_config_display_progress_spinner(struct gcli_ctx *ctx)
 	cfg = ctx_config(ctx);
 
 	if (cfg->no_spinner)
-		return 0;
+		return false;
 
-	sn_sv cfg_entry = gcli_config_find_by_key(ctx, "defaults", "disable-spinner");
-	if (sn_sv_null(cfg_entry))
-		return 1;
+	gcli_sv cfg_entry = gcli_config_find_by_key(ctx, "defaults", "disable-spinner");
+	if (gcli_sv_null(cfg_entry))
+		return true;
 
-	if (checkyes(sn_sv_to_cstr(cfg_entry)))
-		return 0;
+	if (string_means_true(gcli_sv_to_cstr(cfg_entry)))
+		return false;
 
-	return 1;
+	return true;
+}
+
+bool
+gcli_config_render_markdown(struct gcli_ctx *ctx)
+{
+	ensure_config(ctx);
+
+	struct gcli_config *cfg;
+	cfg = ctx_config(ctx);
+
+	if (cfg->no_markdown)
+		return false;
+
+	gcli_sv cfg_entry = gcli_config_find_by_key(ctx, "defaults", "render-markdown");
+	if (gcli_sv_null(cfg_entry))
+		return true;
+
+	if (string_means_false(gcli_sv_to_cstr(cfg_entry))) {
+		cfg->no_markdown = true;
+		return false;
+	}
+
+	return true;
 }
 
 bool
@@ -1027,9 +1110,9 @@ gcli_config_enable_experimental(struct gcli_ctx *ctx)
 	if (cfg->enable_experimental)
 		return true;
 
-	sn_sv cfg_entry = gcli_config_find_by_key(ctx, "defaults", "enable-experimental");
-	if (sn_sv_null(cfg_entry))
+	gcli_sv cfg_entry = gcli_config_find_by_key(ctx, "defaults", "enable-experimental");
+	if (gcli_sv_null(cfg_entry))
 		return false;
 
-	return checkyes(sn_sv_to_cstr(cfg_entry));
+	return string_means_true(gcli_sv_to_cstr(cfg_entry));
 }
