@@ -1,5 +1,5 @@
 /*
- * Copyright 2021, 2022 Nico Sonack <nsonack@herrhotzenplotz.de>
+ * Copyright 2021-2025 Nico Sonack <nsonack@herrhotzenplotz.de>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -27,13 +27,17 @@
  * OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+#include <gcli/cmd/cmd.h>
 #include <gcli/cmd/cmdconfig.h>
 #include <gcli/cmd/gitconfig.h>
 
 #include <gcli/ctx.h>
 #include <gcli/gcli.h>
-#include <sn/sn.h>
+#include <gcli/port/string.h>
+#include <gcli/port/util.h>
+#include <gcli/url.h>
 
+#include <ctype.h>
 #include <dirent.h>
 #include <signal.h>
 #include <stdio.h>
@@ -104,7 +108,7 @@ resolve_worktree_gitdir_if_needed(char *dotgit)
 	 * in .git in a worktree. It contains the path to the real
 	 * .git directory */
 	{
-		char *other_path = sn_asprintf("%s/%s", newdir, "commondir");
+		char *other_path = gcli_asprintf("%s/%s", newdir, "commondir");
 		if ((f = fopen(other_path, "r"))) {
 			char buf[256] = {0};
 			char *tmp = newdir;
@@ -114,7 +118,7 @@ resolve_worktree_gitdir_if_needed(char *dotgit)
 			if (buf[len-1] == '\n')
 				buf[len-1] = '\0';
 
-			newdir = sn_asprintf("%s/%s", tmp, buf);
+			newdir = gcli_asprintf("%s/%s", tmp, buf);
 			free(tmp);
 
 			fclose(f);
@@ -131,7 +135,7 @@ resolve_worktree_gitdir_if_needed(char *dotgit)
  *
  * This is ugly code. However, I don't see an easier way to do
  * this. */
-static char const *
+static char *
 find_file_in_dotgit(char const *fname)
 {
 	DIR           *curr_dir    = NULL;
@@ -184,7 +188,7 @@ find_file_in_dotgit(char const *fname)
 
 			free(curr_dir_path);
 
-			curr_dir_path = realpath(tmp, NULL);
+			curr_dir_path = gcli_cmd_realpath(tmp);
 			if (!curr_dir_path)
 				err(1, "gcli: error: realpath at %s", tmp);
 
@@ -194,7 +198,7 @@ find_file_in_dotgit(char const *fname)
 			if (strcmp("/", curr_dir_path) == 0) {
 				free(curr_dir_path);
 				closedir(curr_dir);
-				warnx("not a git repository");
+				gcli_warnx(g_clictx, "not a git repository");
 				return NULL;
 			}
 		}
@@ -243,18 +247,18 @@ find_file_in_dotgit(char const *fname)
 	return NULL;
 }
 
-char const *
+char *
 gcli_find_gitconfig(void)
 {
 	return find_file_in_dotgit("config");
 }
 
-sn_sv
+gcli_sv
 gcli_gitconfig_get_current_branch(void)
 {
 	char const *HEAD;
-	void       *mmap_pointer;
-	sn_sv       buffer;
+	char       *file_text;
+	gcli_sv     buffer;
 	char        prefix[] = "ref: refs/heads/";
 
 	HEAD = find_file_in_dotgit("HEAD");
@@ -262,143 +266,108 @@ gcli_gitconfig_get_current_branch(void)
 	if (!HEAD)
 		return SV_NULL;
 
-	int len = sn_mmap_file(HEAD, &mmap_pointer);
+	int len = gcli_read_file(HEAD, &file_text);
 	if (len < 0)
 		err(1, "gcli: mmap");
 
-	buffer = sn_sv_from_parts(mmap_pointer, len);
+	buffer = gcli_sv_from_parts(file_text, len);
 
-	if (sn_sv_has_prefix(buffer, prefix)) {
+	if (gcli_sv_has_prefix(buffer, prefix)) {
 		buffer.data   += sizeof(prefix) - 1;
 		buffer.length -= sizeof(prefix) - 1;
 
-		return sn_sv_trim(buffer);
+		return gcli_sv_trim(buffer);
 	} else {
-		munmap(mmap_pointer, len);
+		free(file_text);
 		return SV_NULL;
 	}
 }
 
 static void
-http_extractor(struct gcli_gitremote *const remote, char const *prefix)
+parse_remote_url(struct gcli_gitremote *const remote)
 {
-	size_t prefix_size = strlen(prefix);
-	sn_sv  pair        = remote->url;
+	char *tmp;
+	int rc = 0;
+	size_t n = 0;
+	struct gcli_url url = {0};
 
-	if (sn_sv_has_prefix(remote->url, "https://github.com/")) {
-		prefix_size = sizeof("https://github.com/") - 1;
-		remote->forge_type = GCLI_FORGE_GITHUB;
-	} else if (sn_sv_has_prefix(remote->url, "https://gitlab.com/")) {
-		prefix_size = sizeof("https://gitlab.com/") - 1;
-		remote->forge_type = GCLI_FORGE_GITLAB;
-	} else if (sn_sv_has_prefix(remote->url, "https://codeberg.org/")) {
-		prefix_size = sizeof("https://codeberg.org/") - 1;
-		remote->forge_type = GCLI_FORGE_GITEA;
-	} else {
-		warnx("non-github, non-gitlab and non-codeberg https remotes are "
-		      "not supported and will likely cause bugs");
+	rc = gcli_parse_url(remote->url, &url);
+	if (rc < 0) {
+		fprintf(stderr, "gcli: failed to parse remote url: %s. "
+		        "This is probably a bug.\n", remote->url);
+
+		goto bail;
 	}
 
-	pair.length -= prefix_size;
-	pair.data   += prefix_size;
-
-	remote->owner = sn_sv_chop_until(&pair, '/');
-
-	pair.data   += 1;
-	pair.length -= 1;
-
-	pair = sn_sv_strip_suffix(pair, ".git");
-
-	remote->repo = pair;
-}
-
-static void
-ssh_extractor(struct gcli_gitremote *const remote, char const *prefix)
-{
-	size_t prefix_size = strlen(prefix);
-
-	if (sn_sv_has_prefix(remote->url, "git@github.com"))
+	/* automagic forge type */
+	if (strcmp(url.host, "github.com") == 0)
 		remote->forge_type = GCLI_FORGE_GITHUB;
-	else if (sn_sv_has_prefix(remote->url, "git@gitlab.com"))
+	else if (strcmp(url.host, "gitlab.com") == 0)
 		remote->forge_type = GCLI_FORGE_GITLAB;
-	else if (sn_sv_has_prefix(remote->url, "git@codeberg.org"))
+	else if (strcmp(url.host, "codeberg.org") == 0)
 		remote->forge_type = GCLI_FORGE_GITEA;
 
-	sn_sv pair   = remote->url;
-	pair.length -= prefix_size;
-	pair.data   += prefix_size;
+	tmp = strrchr(url.path, '/');
+	if (tmp == NULL)
+		goto bail;
 
-	sn_sv_chop_until(&pair, ':');
-	pair.data   += 1;
-	pair.length -= 1;
+	remote->owner = gcli_strndup(url.path, tmp - url.path);
 
-	remote->owner = sn_sv_chop_until(&pair, '/');
+	/* skip over '/' */
+	tmp += 1;
 
-	pair.data   += 1;
-	pair.length -= 1;
+	n = strlen(tmp);
+	if (n > 4 && strcmp(tmp + (n - 4), ".git") == 0)
+		n -= 4;
 
-	pair = sn_sv_strip_suffix(pair, ".git");
+	remote->repo = gcli_strndup(tmp, n);
 
-	remote->repo = pair;
+bail:
+	gcli_url_free(&url);
 }
 
-struct forge_ex_def {
-	char const *prefix;
-	void (*extractor)(struct gcli_gitremote *const, char const *);
-} url_extractors[] = {
-	{ .prefix = "git@",     .extractor = ssh_extractor  },
-	{ .prefix = "ssh://",   .extractor = ssh_extractor  },
-	{ .prefix = "https://", .extractor = http_extractor },
-};
-
 static void
-gitconfig_parse_remote(sn_sv section_title, sn_sv entry)
+gitconfig_parse_remote(gcli_sv section_title, gcli_sv entry)
 {
-	sn_sv remote_name = SV_NULL;
+	gcli_sv remote_name = SV_NULL;
 
 	/* If there is no remote name, just return and continue with the
 	 * next section. I don't exactly know why there even are such
 	 * sections and what they are useful for, but ok. */
-	if (sn_sv_eq_to(sn_sv_trim(section_title), "remote"))
+	if (gcli_sv_eq_to(gcli_sv_trim(section_title), "remote"))
 		return;
 
 	/* the remote name is wrapped in double quotes */
-	sn_sv_chop_until(&section_title, '"');
+	gcli_sv_chop_until(&section_title, '"');
 
 	/* skip the first quote */
 	section_title.data   += 1;
 	section_title.length -= 1;
 
-	remote_name = sn_sv_chop_until(&section_title, '"');
+	remote_name = gcli_sv_chop_until(&section_title, '"');
 
-	while ((entry = sn_sv_trim_front(entry)).length > 0) {
-		if (sn_sv_has_prefix(entry, "url")) {
+	while ((entry = gcli_sv_trim_front(entry)).length > 0) {
+		if (gcli_sv_has_prefix(entry, "url")) {
 			if (remotes_size == MAX_REMOTES)
 				errx(1, "gcli: error: too many remotes");
 
 			struct gcli_gitremote *const remote = &remotes[remotes_size++];
 
-			remote->name = remote_name;
+			remote->name = gcli_sv_to_cstr(remote_name);
 
-			sn_sv_chop_until(&entry, '=');
+			gcli_sv_chop_until(&entry, '=');
 
 			entry.data   += 1;
 			entry.length -= 1;
 
-			sn_sv url = sn_sv_trim(sn_sv_chop_until(&entry, '\n'));
+			gcli_sv url = gcli_sv_trim(gcli_sv_chop_until(&entry, '\n'));
 
-			remote->url        = url;
+			remote->url = gcli_sv_to_cstr(url);
 			remote->forge_type = -1;
 
-			for (size_t i = 0; i < ARRAY_SIZE(url_extractors); ++i) {
-				if (sn_sv_has_prefix(url, url_extractors[i].prefix)) {
-					url_extractors[i].extractor(
-						remote,
-						url_extractors[i].prefix);
-				}
-			}
+			parse_remote_url(remote);
 		} else {
-			sn_sv_chop_until(&entry, '\n');
+			gcli_sv_chop_until(&entry, '\n');
 		}
 	}
 }
@@ -406,6 +375,8 @@ gitconfig_parse_remote(sn_sv section_title, sn_sv entry)
 static void
 gcli_gitconfig_read_gitconfig(void)
 {
+	char *path = NULL;
+	gcli_sv buffer = {0}, filebuf = {0};
 	static int has_read_gitconfig = 0;
 
 	if (has_read_gitconfig)
@@ -413,17 +384,18 @@ gcli_gitconfig_read_gitconfig(void)
 
 	has_read_gitconfig = 1;
 
-	char const *path   = NULL;
-	sn_sv       buffer = {0};
-
 	path = gcli_find_gitconfig();
 	if (!path)
 		return;
 
-	buffer.length = sn_mmap_file(path, (void **)&buffer.data);
+	filebuf.length = gcli_read_file(path, &filebuf.data);
+	buffer = filebuf;
+
+	free(path);
+	path = NULL;
 
 	while (buffer.length > 0) {
-		buffer = sn_sv_trim_front(buffer);
+		buffer = gcli_sv_trim_front(buffer);
 
 		if (buffer.length == 0)
 			break;
@@ -432,21 +404,25 @@ gcli_gitconfig_read_gitconfig(void)
 		if (*buffer.data != '[')
 			errx(1, "gcli: error: invalid git config");
 
-		sn_sv section_title = sn_sv_chop_until(&buffer, ']');
+		gcli_sv section_title = gcli_sv_chop_until(&buffer, ']');
 		section_title.length -= 1;
 		section_title.data   += 1;
 
 		buffer.length -= 2;
 		buffer.data   += 2;
 
-		sn_sv entry = sn_sv_chop_until(&buffer, '[');
+		gcli_sv entry = gcli_sv_chop_until(&buffer, '[');
 
-		if (sn_sv_has_prefix(section_title, "remote")) {
+		if (gcli_sv_has_prefix(section_title, "remote")) {
 			gitconfig_parse_remote(section_title, entry);
 		} else {
 			// @@@: skip section
 		}
 	}
+
+	free(filebuf.data);
+	filebuf.length = 0;
+	filebuf.data = NULL;
 }
 
 void
@@ -489,7 +465,7 @@ gcli_gitconfig_add_fork_remote(char const *org, char const *repo)
 	{
 		pid_t pid = 0;
 		if ((pid = fork()) == 0) {
-			char const *remote_url = sn_asprintf(
+			char const *remote_url = gcli_asprintf(
 				"git@github.com:%s/%s",
 				org, repo);
 			printf("[INFO] git remote add origin %s\n", remote_url);
@@ -517,13 +493,13 @@ gcli_gitconfig_get_forgetype(struct gcli_ctx *ctx, char const *const remote_name
 
 	if (remote_name) {
 		for (size_t i = 0; i < remotes_size; ++i) {
-			if (sn_sv_eq_to(remotes[i].name, remote_name))
+			if (strcmp(remotes[i].name, remote_name) == 0)
 				return remotes[i].forge_type;
 		}
 	}
 
 	if (!remotes_size) {
-		warn("no remotes to auto-detect forge");
+		gcli_warn(ctx, "no remotes to auto-detect forge");
 		return -1;
 	}
 
@@ -539,9 +515,9 @@ gcli_gitconfig_repo_by_remote(struct gcli_ctx *ctx, char const *const remote,
 
 	if (remote) {
 		for (size_t i = 0; i < remotes_size; ++i) {
-			if (sn_sv_eq_to(remotes[i].name, remote)) {
-				*owner = sn_sv_to_cstr(remotes[i].owner);
-				*repo  = sn_sv_to_cstr(remotes[i].repo);
+			if (strcmp(remotes[i].name, remote) == 0) {
+				*owner = remotes[i].owner;
+				*repo  = remotes[i].repo;
 				if (forge)
 					*forge = remotes[i].forge_type;
 
@@ -555,10 +531,26 @@ gcli_gitconfig_repo_by_remote(struct gcli_ctx *ctx, char const *const remote,
 	if (!remotes_size)
 		return gcli_error(ctx, "no remotes to auto-detect forge");
 
-	*owner = sn_sv_to_cstr(remotes[0].owner);
-	*repo  = sn_sv_to_cstr(remotes[0].repo);
+	*owner = remotes[0].owner;
+	*repo  = remotes[0].repo;
 	if (forge)
 		*forge = remotes[0].forge_type;
 
 	return 0;
 }
+
+int
+gcli_gitconfig_get_remote(struct gcli_ctx *ctx, gcli_forge_type const type,
+                          char const **remote)
+{
+	gcli_gitconfig_read_gitconfig();
+
+	for (size_t i = 0; i < remotes_size; ++i) {
+		if (remotes[i].forge_type == type) {
+			*remote = remotes[i].url;
+			return 0;
+		}
+	}
+
+	return gcli_error(ctx, "no suitable remote for forge type");
+}
