@@ -34,10 +34,7 @@
 
 #include <gcli/gitlab/snippets.h>
 
-#ifdef HAVE_GETOPT_H
 #include <getopt.h>
-#endif
-
 #include <stdlib.h>
 
 static void
