@@ -1,5 +1,5 @@
 /*
- * Copyright 2023 Nico Sonack <nsonack@herrhotzenplotz.de>
+ * Copyright 2023-2025 Nico Sonack <nsonack@herrhotzenplotz.de>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -30,9 +30,11 @@
 #include <gcli/forges.h>
 #include <gcli/gcli.h>
 
+#include <errno.h>
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 
 int
 gcli_error(struct gcli_ctx *ctx, char const *const fmt, ...)
@@ -52,7 +54,7 @@ gcli_error(struct gcli_ctx *ctx, char const *const fmt, ...)
 	va_end(vp);
 
 	if (ctx->last_error)
-		free(ctx->last_error);
+		gcli_clear_ptr(&ctx->last_error);
 
 	ctx->last_error = buf;
 
@@ -103,7 +105,63 @@ gcli_get_authheader(struct gcli_ctx *ctx)
 		hdr = gcli_forge(ctx)->make_authheader(ctx, token);
 	}
 
-	free(token);
+	gcli_clear_ptr(&token);
 
 	return hdr;
 }
+
+bool
+gcli_be_verbose(struct gcli_ctx *ctx)
+{
+	return ctx->verbosity == GCLI_VERBOSITY_VERBOSE;
+}
+
+bool
+gcli_be_quiet(struct gcli_ctx *ctx)
+{
+	return ctx->verbosity == GCLI_VERBOSITY_QUIET;
+}
+
+int
+gcli_getverbosity(struct gcli_ctx *ctx)
+{
+	return ctx->verbosity;
+}
+
+void
+gcli_setverbosity(struct gcli_ctx *ctx, int v)
+{
+	ctx->verbosity = v;
+}
+
+void
+gcli_warn(struct gcli_ctx *ctx, char const *fmt, ...)
+{
+	if (!gcli_be_verbose(ctx))
+		return;
+
+	fputs("warning: ", stderr);
+	va_list ap;
+
+	va_start(ap, fmt);
+	vfprintf(stderr, fmt, ap);
+	va_end(ap);
+
+	fprintf(stderr, ": %s\n", strerror(errno));
+}
+
+void
+gcli_warnx(struct gcli_ctx *ctx, char const *fmt, ...)
+{
+	if (!gcli_be_verbose(ctx))
+		return;
+
+	fputs("warning: ", stderr);
+	va_list ap;
+
+	va_start(ap, fmt);
+	vfprintf(stderr, fmt, ap);
+	va_end(ap);
+
+	fputc('\n', stderr);
+}
