@@ -1,5 +1,5 @@
 /*
- * Copyright 2023 Nico Sonack <nsonack@herrhotzenplotz.de>
+ * Copyright 2023-2025 Nico Sonack <nsonack@herrhotzenplotz.de>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -28,8 +28,8 @@
  */
 
 #include <gcli/diffutil.h>
-
-#include <sn/sn.h>
+#include <gcli/gcli.h>
+#include <gcli/port/string.h>
 
 #include <assert.h>
 #include <string.h>
@@ -127,7 +127,7 @@ gcli_parse_patch(struct gcli_diff_parser *parser, struct gcli_patch *out)
 
 	/* TODO cleanup */
 	while (parser->hd[0] == 'd') {
-		struct gcli_diff *d = calloc(sizeof(*d), 1);
+		struct gcli_diff *d = calloc(1, sizeof(*d));
 
 		if (gcli_parse_diff(parser, d) < 0)
 			return -1;
@@ -178,7 +178,7 @@ read_commit_hash_from_separator(struct token const *line, struct gcli_patch *out
 	if (!end_of_hash)
 		return -1;
 
-	out->commit_hash = sn_strndup(start_of_hash, end_of_hash - start_of_hash);
+	out->commit_hash = gcli_strndup(start_of_hash, end_of_hash - start_of_hash);
 	return 0;
 }
 
@@ -349,8 +349,9 @@ parse_hunk_range_info(struct gcli_diff_parser *parser,
 	if (read_number(&line, 10, &out->range_r_start) < 0)
 		return -1;
 
-	char const delim_r = *line.start++;
+	char const delim_r = *line.start;
 	if (delim_r == ',') {
+		line.start += 1;
 		if (read_number(&line, 10, &out->range_r_length) < 0)
 			return -1;
 	} else if (delim_r != ' ') {
@@ -522,7 +523,7 @@ read_hunk_body(struct gcli_diff_parser *parser, struct gcli_diff_hunk *hunk)
 		/* If it is a comment, don't count this line into the absolute diff
 		 * offset of the hunk */
 		if (line.start[0] == ' ' || line.start[0] == '+' ||
-		    line.start[0] == '-')
+		    line.start[0] == '-' || line.start[0] == '\\')
 		{
 			parser->diff_line_offset += 1;
 		}
@@ -620,15 +621,15 @@ gcli_parse_diff(struct gcli_diff_parser *parser, struct gcli_diff *out)
 	parser->diff_line_offset = 0;
 	TAILQ_INIT(&out->hunks);
 	while (parser->hd[0] == '@') {
-		struct gcli_diff_hunk *hunk = calloc(sizeof(*hunk), 1);
+		struct gcli_diff_hunk *hunk = calloc(1, sizeof(*hunk));
 
 		if (parse_hunk_range_info(parser, hunk) < 0) {
-			free(hunk);
+			gcli_clear_ptr(&hunk);
 			return -1;
 		}
 
 		if (read_hunk_body(parser, hunk) < 0) {
-			free(hunk);
+			gcli_clear_ptr(&hunk);
 			return -1;
 		}
 
@@ -674,7 +675,7 @@ gcli_parse_patch_series(struct gcli_diff_parser *parser,
 		return -1;
 
 	while (parser->hd[0] != '\0') {
-		struct gcli_patch *p = calloc(sizeof(*p), 1);
+		struct gcli_patch *p = calloc(1, sizeof(*p));
 
 		TAILQ_INSERT_TAIL(&series->patches, p, next);
 		if (gcli_parse_patch(parser, p) < 0)
@@ -687,42 +688,26 @@ gcli_parse_patch_series(struct gcli_diff_parser *parser,
 void
 gcli_free_diff_hunk(struct gcli_diff_hunk *hunk)
 {
-	free(hunk->context_info);
-	hunk->context_info = NULL;
-
-	free(hunk->body);
-	hunk->body = NULL;
+	gcli_clear_ptr(&hunk->context_info);
+	gcli_clear_ptr(&hunk->body);
 }
 
 void
 gcli_free_diff(struct gcli_diff *diff)
 {
-	free(diff->file_a);
-	diff->file_a = NULL;
-
-	free(diff->file_b);
-	diff->file_b = NULL;
-
-	free(diff->hash_a);
-	diff->hash_a = NULL;
-
-	free(diff->hash_b);
-	diff->hash_b = NULL;
-
-	free(diff->file_mode);
-	diff->file_mode = NULL;
-
-	free(diff->r_file);
-	diff->r_file = NULL;
-
-	free(diff->a_file);
-	diff->a_file = NULL;
+	gcli_clear_ptr(&diff->file_a);
+	gcli_clear_ptr(&diff->file_b);
+	gcli_clear_ptr(&diff->hash_a);
+	gcli_clear_ptr(&diff->hash_b);
+	gcli_clear_ptr(&diff->file_mode);
+	gcli_clear_ptr(&diff->r_file);
+	gcli_clear_ptr(&diff->a_file);
 
 	struct gcli_diff_hunk *h = TAILQ_FIRST(&diff->hunks);
 	while (h) {
 		struct gcli_diff_hunk *n = TAILQ_NEXT(h, next);
 		gcli_free_diff_hunk(h);
-		free(h);
+		gcli_clear_ptr(&h);
 		h = n;
 	}
 	TAILQ_INIT(&diff->hunks);
@@ -733,14 +718,13 @@ gcli_free_patch(struct gcli_patch *patch)
 {
 	struct gcli_diff *d, *n;
 
-	free(patch->prelude);
-	patch->prelude = NULL;
+	gcli_clear_ptr(&patch->prelude);
 
 	d = TAILQ_FIRST(&patch->diffs);
 	while (d) {
 		n = TAILQ_NEXT(d, next);
 		gcli_free_diff(d);
-		free(d);
+		gcli_clear_ptr(&d);
 		d = n;
 	}
 	TAILQ_INIT(&patch->diffs);
@@ -750,7 +734,7 @@ void
 gcli_free_diff_parser(struct gcli_diff_parser *parser)
 {
 	if (parser->buf_needs_free)
-		free((char *)parser->buf);
+		gcli_clear_ptr(&parser->buf);
 
 	memset(parser, 0, sizeof(*parser));
 }
@@ -777,7 +761,7 @@ static struct gcli_diff_comment *
 make_comment(struct comment_read_ctx *ctx, char *text,
              struct hunk_line_info const *line_info, int diff_line_offset)
 {
-	struct gcli_diff_comment *comment = calloc(sizeof(*comment), 1);
+	struct gcli_diff_comment *comment = calloc(1, sizeof(*comment));
 	comment->after.filename = strdup(ctx->diff->file_b);
 	comment->after.start_row = line_info->patched_line;
 	comment->after.end_row = line_info->patched_line;
@@ -924,6 +908,7 @@ gcli_hunk_get_comments(struct gcli_diff const *diff,
 		case '+':
 		case ' ':
 		case '-':
+		case '\\':
 			ctx.diff_line_offset += 1;
 			ctx.last_line_is_new = hd == '+';
 
@@ -1056,7 +1041,7 @@ gcli_free_patch_series(struct gcli_patch_series *series)
 	while (p) {
 		struct gcli_patch *n = TAILQ_NEXT(p, next);
 		gcli_free_patch(p);
-		free(p);
+		gcli_clear_ptr(&p);
 		p = n;
 	}
 	TAILQ_INIT(&series->patches);
