@@ -1,5 +1,5 @@
 /*
- * Copyright 2021, 2022, 2023 Nico Sonack <nsonack@herrhotzenplotz.de>
+ * Copyright 2021-2025 Nico Sonack <nsonack@herrhotzenplotz.de>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -32,6 +32,7 @@
 #include <gcli/forges.h>
 
 #include <gcli/github/api.h>
+#include <gcli/github/checkout.h>
 #include <gcli/github/comments.h>
 #include <gcli/github/config.h>
 #include <gcli/github/forks.h>
@@ -45,6 +46,7 @@
 #include <gcli/github/status.h>
 
 #include <gcli/gitlab/api.h>
+#include <gcli/gitlab/checkout.h>
 #include <gcli/gitlab/comments.h>
 #include <gcli/gitlab/config.h>
 #include <gcli/gitlab/forks.h>
@@ -73,12 +75,16 @@
 #include <gcli/bugzilla/api.h>
 #include <gcli/bugzilla/attachments.h>
 #include <gcli/bugzilla/bugs.h>
+#include <gcli/bugzilla/comment.h>
 #include <gcli/bugzilla/config.h>
 
+#include <gcli/port/err.h>
+
 static struct gcli_forge_descriptor const
 github_forge_descriptor =
 {
 	/* Comments */
+	.get_comment               = github_get_comment,
 	.get_issue_comments        = github_get_comments,
 	.get_pull_comments         = github_get_comments,
 	.perform_submit_comment    = github_perform_submit_comment,
@@ -98,10 +104,10 @@ github_forge_descriptor =
 	.issue_reopen              = github_issue_reopen,
 	.issue_set_milestone       = github_issue_set_milestone,
 	.issue_set_title           = github_issue_set_title,
+	.issue_set_op              = github_issue_set_op,
 	.perform_submit_issue      = github_perform_submit_issue,
 	.issue_quirks              = GCLI_ISSUE_QUIRKS_PROD_COMP
-	                           | GCLI_ISSUE_QUIRKS_URL
-	                           | GCLI_ISSUE_QUIRKS_ATTACHMENTS,
+	                           | GCLI_ISSUE_QUIRKS_URL,
 
 	/* Milestones */
 	.create_milestone          = github_create_milestone,
@@ -118,12 +124,14 @@ github_forge_descriptor =
 	.search_pulls              = github_search_pulls,
 	.perform_submit_pull       = github_perform_submit_pull,
 	.pull_add_reviewer         = github_pull_add_reviewer,
+	.pull_assign               = github_issue_assign,
 	.pull_close                = github_pull_close,
 	.pull_get_diff             = github_pull_get_diff,
 	.pull_get_patch            = github_pull_get_patch,
 	.pull_merge                = github_pull_merge,
 	.pull_reopen               = github_pull_reopen,
 	.pull_set_title            = github_pull_set_title,
+	.pull_checkout             = github_pull_checkout,
 
 	/* HACK: Here we can use the same functions as with issues because
 	 * PRs are the same as issues on Github and the functions have the
@@ -134,6 +142,8 @@ github_forge_descriptor =
 	.pull_set_milestone        = github_issue_set_milestone,
 
 	.pull_create_review        = github_pull_create_review,
+	.pull_get_reviews          = github_pull_get_reviews,
+	.pull_get_review_threads   = github_pull_get_review_threads,
 
 	.create_release            = github_create_release,
 	.delete_release            = github_delete_release,
@@ -143,6 +153,10 @@ github_forge_descriptor =
 	.create_label              = github_create_label,
 	.delete_label              = github_delete_label,
 	.get_labels                = github_get_labels,
+	.get_label                 = github_get_label,
+	.label_set_title           = github_label_set_title,
+	.label_set_description     = github_label_set_description,
+	.label_set_colour          = github_label_set_colour,
 
 	/* Repos */
 	.get_repos                 = github_get_repos,
@@ -158,8 +172,6 @@ github_forge_descriptor =
 	/* Notifications */
 	.get_notifications         = github_get_notifications,
 	.notification_mark_as_read = github_notification_mark_as_read,
-	.notification_get_issue    = github_notification_get_issue,
-	.notification_get_comments = github_notification_get_comments,
 
 	/* Internal stuff */
 	.get_api_error_string      = github_api_error_string,
@@ -171,13 +183,14 @@ github_forge_descriptor =
 	                           | GCLI_MILESTONE_QUIRKS_DUEDATE
 	                           | GCLI_MILESTONE_QUIRKS_PULLS,
 	.pull_summary_quirks       = GCLI_PRS_QUIRK_COVERAGE
-	                           | GCLI_PRS_QUIRK_AUTOMERGE,
+	                           | GCLI_PRS_QUIRK_AUTOMERGE, /* automerge field seems broken */
 };
 
 static struct gcli_forge_descriptor const
 gitlab_forge_descriptor =
 {
 	/* Comments */
+	.get_comment               = gitlab_get_comment,
 	.get_issue_comments        = gitlab_get_issue_comments,
 	.get_pull_comments         = gitlab_get_mr_comments,
 	.perform_submit_comment    = gitlab_perform_submit_comment,
@@ -197,10 +210,10 @@ gitlab_forge_descriptor =
 	.issue_reopen              = gitlab_issue_reopen,
 	.issue_set_milestone       = gitlab_issue_set_milestone,
 	.issue_set_title           = gitlab_issue_set_title,
+	.issue_set_op              = gitlab_issue_set_op,
 	.perform_submit_issue      = gitlab_perform_submit_issue,
 	.issue_quirks              = GCLI_ISSUE_QUIRKS_PROD_COMP
-	                           | GCLI_ISSUE_QUIRKS_URL
-	                           | GCLI_ISSUE_QUIRKS_ATTACHMENTS,
+	                           | GCLI_ISSUE_QUIRKS_URL,
 
 	/* Milestones */
 	.create_milestone          = gitlab_create_milestone,
@@ -218,6 +231,7 @@ gitlab_forge_descriptor =
 	.perform_submit_pull       = gitlab_perform_submit_mr,
 	.pull_add_labels           = gitlab_mr_add_labels,
 	.pull_add_reviewer         = gitlab_mr_add_reviewer,
+	.pull_assign               = gitlab_mr_add_assignee,
 	.pull_clear_milestone      = gitlab_mr_clear_milestone,
 	.pull_close                = gitlab_mr_close,
 	.pull_create_review        = gitlab_mr_create_review,
@@ -228,6 +242,7 @@ gitlab_forge_descriptor =
 	.pull_reopen               = gitlab_mr_reopen,
 	.pull_set_milestone        = gitlab_mr_set_milestone,
 	.pull_set_title            = gitlab_mr_set_title,
+	.pull_checkout             = gitlab_mr_checkout,
 
 	/* Releases */
 	.create_release            = gitlab_create_release,
@@ -238,6 +253,10 @@ gitlab_forge_descriptor =
 	.create_label              = gitlab_create_label,
 	.delete_label              = gitlab_delete_label,
 	.get_labels                = gitlab_get_labels,
+	.get_label                 = gitlab_get_label,
+	.label_set_title           = gitlab_label_set_title,
+	.label_set_description     = gitlab_label_set_description,
+	.label_set_colour          = gitlab_label_set_colour,
 
 	/* Repos */
 	.get_repos                 = gitlab_get_repos,
@@ -252,9 +271,7 @@ gitlab_forge_descriptor =
 
 	/* Notifications */
 	.get_notifications         = gitlab_get_notifications,
-	.notification_get_issue    = gitlab_notification_get_issue,
 	.notification_mark_as_read = gitlab_notification_mark_as_read,
-	.notification_get_comments = gitlab_notification_get_comments,
 
 	/* Internal stuff */
 	.get_api_error_string      = gitlab_api_error_string,
@@ -273,6 +290,7 @@ static struct gcli_forge_descriptor const
 gitea_forge_descriptor =
 {
 	/* Comments */
+	.get_comment               = gitea_get_comment,
 	.get_issue_comments        = gitea_get_comments,
 	.get_pull_comments         = gitea_get_comments,
 	.perform_submit_comment    = gitea_perform_submit_comment,
@@ -292,10 +310,10 @@ gitea_forge_descriptor =
 	.issue_reopen              = gitea_issue_reopen,
 	.issue_set_milestone       = gitea_issue_set_milestone,
 	.issue_set_title           = gitea_issue_set_title,
+	.issue_set_op              = gitea_issue_set_op,
 	.perform_submit_issue      = gitea_submit_issue,
 	.issue_quirks              = GCLI_ISSUE_QUIRKS_PROD_COMP
-	                           | GCLI_ISSUE_QUIRKS_URL
-	                           | GCLI_ISSUE_QUIRKS_ATTACHMENTS,
+	                           | GCLI_ISSUE_QUIRKS_URL,
 
 	/* Milestones */
 	.create_milestone          = gitea_create_milestone,
@@ -313,6 +331,7 @@ gitea_forge_descriptor =
 	.perform_submit_pull       = gitea_pull_submit,
 	.pull_add_labels           = gitea_issue_add_labels,
 	.pull_add_reviewer         = gitea_pull_add_reviewer,
+	.pull_assign               = gitea_pull_assign,
 	.pull_clear_milestone      = gitea_pull_clear_milestone,
 	.pull_close                = gitea_pull_close,
 	.pull_get_diff             = gitea_pull_get_diff,
@@ -322,6 +341,7 @@ gitea_forge_descriptor =
 	.pull_reopen               = gitea_pull_reopen,
 	.pull_set_milestone        = gitea_pull_set_milestone,
 	.pull_set_title            = gitea_pull_set_title,
+	.pull_get_reviews          = gitea_pull_get_reviews,
 
 	/* Releases */
 	.create_release            = gitea_create_release,
@@ -332,6 +352,10 @@ gitea_forge_descriptor =
 	.create_label              = gitea_create_label,
 	.delete_label              = gitea_delete_label,
 	.get_labels                = gitea_get_labels,
+	.get_label                 = gitea_get_label,
+	.label_set_title           = gitea_label_set_title,
+	.label_set_description     = gitea_label_set_description,
+	.label_set_colour          = gitea_label_set_colour,
 
 	/* Repos */
 	.get_repos                 = gitea_get_repos,
@@ -362,6 +386,8 @@ gitea_forge_descriptor =
 	                           | GCLI_PRS_QUIRK_DRAFT
 	                           | GCLI_PRS_QUIRK_CHANGES
 	                           | GCLI_PRS_QUIRK_COVERAGE,
+	.pull_quirks               = GCLI_PULL_QUIRK_AUTOMERGE, /* uses Github Backend with GraphQL
+	                                                         * which doesn't work with gitea */
 };
 
 static struct gcli_forge_descriptor const
@@ -371,11 +397,14 @@ bugzilla_forge_descriptor =
 	.search_issues             = bugzilla_get_bugs,
 	.get_issue_summary         = bugzilla_get_bug,
 	.get_issue_comments        = bugzilla_bug_get_comments,
+	.get_comment               = bugzilla_bug_get_comment,
 	.get_issue_attachments     = bugzilla_bug_get_attachments,
 	.perform_submit_issue      = bugzilla_bug_submit,
 	.issue_quirks              = GCLI_ISSUE_QUIRKS_COMMENTS
 	                           | GCLI_ISSUE_QUIRKS_LOCKED,
 
+	.perform_submit_comment    = bugzilla_submit_comment,
+
 	.attachment_get_content    = bugzilla_attachment_get_content,
 
 	/* Internal stuff */
