@@ -1,5 +1,5 @@
 /*
- * Copyright 2021,2022 Nico Sonack <nsonack@herrhotzenplotz.de>
+ * Copyright 2021-2025 Nico Sonack <nsonack@herrhotzenplotz.de>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -34,25 +34,25 @@
 #include <gcli/github/forks.h>
 
 int
-gcli_get_forks(struct gcli_ctx *ctx, char const *owner, char const *repo,
+gcli_get_forks(struct gcli_ctx *ctx, struct gcli_path const *const path,
                int const max, struct gcli_fork_list *const out)
 {
-	gcli_null_check_call(get_forks, ctx, owner, repo, max, out);
+	gcli_null_check_call(get_forks, ctx, path, max, out);
 }
 
 int
-gcli_fork_create(struct gcli_ctx *ctx, char const *owner, char const *repo,
-                 char const *_in)
+gcli_fork_create(struct gcli_ctx *ctx,
+                 struct gcli_path const *const repo_path,
+                 char const *const in)
 {
-	gcli_null_check_call(fork_create, ctx, owner, repo, _in);
+	gcli_null_check_call(fork_create, ctx, repo_path, in);
 }
 
 void
 gcli_fork_free(struct gcli_fork *fork)
 {
-	free(fork->full_name);
-	free(fork->owner);
-	free(fork->date);
+	gcli_clear_ptr(&fork->full_name);
+	gcli_clear_ptr(&fork->owner);
 }
 
 void
@@ -62,8 +62,7 @@ gcli_forks_free(struct gcli_fork_list *const list)
 		gcli_fork_free(&list->forks[i]);
 	}
 
-	free(list->forks);
+	gcli_clear_ptr(&list->forks);
 
-	list->forks = NULL;
 	list->forks_size = 0;
 }
