@@ -1,5 +1,5 @@
 /*
- * Copyright 2023 Nico Sonack <nsonack@herrhotzenplotz.de>
+ * Copyright 2023-2025 Nico Sonack <nsonack@herrhotzenplotz.de>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -39,7 +39,7 @@ gcli_init(struct gcli_ctx **ctx,
           char *(*get_token)(struct gcli_ctx *),
           char *(*get_apibase)(struct gcli_ctx *))
 {
-	*ctx = calloc(sizeof (struct gcli_ctx), 1);
+	*ctx = calloc(1, sizeof (struct gcli_ctx));
 	if (!(*ctx))
 		return strerror(errno);
 
@@ -56,10 +56,9 @@ void
 gcli_destroy(struct gcli_ctx **ctx)
 {
 	if (ctx && *ctx) {
-		free((*ctx)->apibase);
+		gcli_clear_ptr(&(*ctx)->apibase);
 
-		free(*ctx);
-		*ctx = NULL;
+		gcli_clear_ptr(ctx);
 
 		/* TODO: other deinit stuff? */
 	}
@@ -73,3 +72,12 @@ gcli_get_error(struct gcli_ctx *ctx)
 	else
 		return "No error";
 }
+
+void
+gcli_clear_ptr(void *ptr)
+{
+	void **_ptr = ptr;
+
+	free(*_ptr);
+	*_ptr = NULL;
+}
