@@ -1,5 +1,5 @@
 /*
- * Copyright 2023 Nico Sonack <nsonack@herrhotzenplotz.de>
+ * Copyright 2023-2025 Nico Sonack <nsonack@herrhotzenplotz.de>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -30,8 +30,7 @@
 #include <gcli/curl.h>
 #include <gcli/gitea/status.h>
 #include <gcli/github/status.h>
-
-#include <sn/sn.h>
+#include <gcli/port/string.h>
 
 #include <templates/gitea/status.h>
 
@@ -48,7 +47,8 @@ gitea_get_notifications(struct gcli_ctx *ctx, int const max,
 		.max = max,
 	};
 
-	url = sn_asprintf("%s/notifications", gcli_get_apibase(ctx));
+	url = gcli_asprintf("%s/notifications", gcli_get_apibase(ctx));
+
 	return gcli_fetch_list(ctx, url, &fl);
 }
 
