@@ -1,5 +1,5 @@
 /*
- * Copyright 2021, 2022 Nico Sonack <nsonack@herrhotzenplotz.de>
+ * Copyright 2021-2025 Nico Sonack <nsonack@herrhotzenplotz.de>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -36,7 +36,7 @@
 
 #include <gcli/github/config.h>
 
-#include <pdjson/pdjson.h>
+#include <pdjson.h>
 
 #include <templates/github/gists.h>
 
@@ -52,7 +52,7 @@ parse_github_gist_files_idiot_hack(struct gcli_ctx *ctx, json_stream *stream,
 	gist->files = NULL;
 	gist->files_size = 0;
 
-	if ((next = json_next(stream)) != JSON_OBJECT)
+	if (json_next(stream) != JSON_OBJECT)
 		return gcli_error(ctx, "expected Gist Files Object");
 
 	while ((next = json_next(stream)) == JSON_STRING) {
@@ -81,9 +81,9 @@ gcli_get_gists(struct gcli_ctx *ctx, char const *user, int const max,
 	};
 
 	if (user)
-		url = sn_asprintf("%s/users/%s/gists", gcli_get_apibase(ctx), user);
+		url = gcli_asprintf("%s/users/%s/gists", gcli_get_apibase(ctx), user);
 	else
-		url = sn_asprintf("%s/gists", gcli_get_apibase(ctx));
+		url = gcli_asprintf("%s/gists", gcli_get_apibase(ctx));
 
 	return gcli_fetch_list(ctx, url, &fl);
 }
@@ -95,7 +95,7 @@ gcli_get_gist(struct gcli_ctx *ctx, char const *gist_id, struct gcli_gist *out)
 	struct gcli_fetch_buffer buffer = {0};
 	int rc = 0;
 
-	url = sn_asprintf("%s/gists/%s", gcli_get_apibase(ctx), gist_id);
+	url = gcli_asprintf("%s/gists/%s", gcli_get_apibase(ctx), gist_id);
 	rc = gcli_fetch(ctx, url, NULL, &buffer);
 
 	if (rc == 0) {
@@ -110,7 +110,7 @@ gcli_get_gist(struct gcli_ctx *ctx, char const *gist_id, struct gcli_gist *out)
 	}
 
 	gcli_fetch_buffer_free(&buffer);
-	free(url);
+	gcli_clear_ptr(&url);
 
 	return rc;
 }
@@ -137,7 +137,7 @@ read_file(FILE *f)
 	}
 
 	if (ferror(f)) {
-		free(out);
+		gcli_clear_ptr(&out);
 		out = NULL;
 	}
 
@@ -205,15 +205,15 @@ gcli_create_gist(struct gcli_ctx *ctx, struct gcli_new_gist opts)
 	gcli_jsongen_free(&gen);
 
 	/* Generate URL */
-	url = sn_asprintf("%s/gists", gcli_get_apibase(ctx));
+	url = gcli_asprintf("%s/gists", gcli_get_apibase(ctx));
 
 	/* Perferm fetch */
 	rc = gcli_fetch_with_method(ctx, "POST", url, post_data, NULL, &buffer);
 
 	gcli_fetch_buffer_free(&buffer);
-	free(content);
-	free(url);
-	free(post_data);
+	gcli_clear_ptr(&content);
+	gcli_clear_ptr(&url);
+	gcli_clear_ptr(&post_data);
 
 	return rc;
 }
@@ -225,12 +225,12 @@ gcli_delete_gist(struct gcli_ctx *ctx, char const *gist_id)
 	struct gcli_fetch_buffer buffer = {0};
 	int rc = 0;
 
-	url = sn_asprintf("%s/gists/%s", gcli_get_apibase(ctx), gist_id);
+	url = gcli_asprintf("%s/gists/%s", gcli_get_apibase(ctx), gist_id);
 
 	rc = gcli_fetch_with_method(ctx, "DELETE", url, NULL, NULL, &buffer);
 
 	gcli_fetch_buffer_free(&buffer);
-	free(url);
+	gcli_clear_ptr(&url);
 
 	return rc;
 }
@@ -238,21 +238,21 @@ gcli_delete_gist(struct gcli_ctx *ctx, char const *gist_id)
 void
 gcli_gist_free(struct gcli_gist *g)
 {
-	free(g->id);
-	free(g->owner);
-	free(g->url);
-	free(g->date);
-	free(g->git_pull_url);
-	free(g->description);
+	gcli_clear_ptr(&g->id);
+	gcli_clear_ptr(&g->owner);
+	gcli_clear_ptr(&g->url);
+	gcli_clear_ptr(&g->date);
+	gcli_clear_ptr(&g->git_pull_url);
+	gcli_clear_ptr(&g->description);
 
 	for (size_t j = 0; j < g->files_size; ++j) {
-		free(g->files[j].filename);
-		free(g->files[j].language);
-		free(g->files[j].url);
-		free(g->files[j].type);
+		gcli_clear_ptr(&g->files[j].filename);
+		gcli_clear_ptr(&g->files[j].language);
+		gcli_clear_ptr(&g->files[j].url);
+		gcli_clear_ptr(&g->files[j].type);
 	}
 
-	free(g->files);
+	gcli_clear_ptr(&g->files);
 
 	memset(g, 0, sizeof(*g));
 }
@@ -263,8 +263,6 @@ gcli_gists_free(struct gcli_gist_list *const list)
 	for (size_t i = 0; i < list->gists_size; ++i)
 		gcli_gist_free(&list->gists[i]);
 
-	free(list->gists);
-
-	list->gists = NULL;
+	gcli_clear_ptr(&list->gists);
 	list->gists_size = 0;
 }
