@@ -1,5 +1,5 @@
 /*
- * Copyright 2021, 2022 Nico Sonack <nsonack@herrhotzenplotz.de>
+ * Copyright 2021-2025 Nico Sonack <nsonack@herrhotzenplotz.de>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -30,12 +30,14 @@
 #include <gcli/curl.h>
 #include <gcli/gitlab/config.h>
 #include <gcli/gitlab/releases.h>
+#include <gcli/gitlab/repos.h>
 #include <gcli/json_gen.h>
 #include <gcli/json_util.h>
+#include <gcli/port/err.h>
 
 #include <templates/gitlab/releases.h>
 
-#include <pdjson/pdjson.h>
+#include <pdjson.h>
 
 #include <string.h>
 
@@ -62,12 +64,11 @@ fixup_release_asset_names(struct gcli_ctx *ctx, struct gcli_release_list *list)
 }
 
 int
-gitlab_get_releases(struct gcli_ctx *ctx, char const *owner, char const *repo,
-                    int const max, struct gcli_release_list *const list)
+gitlab_get_releases(struct gcli_ctx *ctx,
+                    struct gcli_path const *const repo_path, int const max,
+                    struct gcli_release_list *const list)
 {
 	char *url = NULL;
-	char *e_owner = NULL;
-	char *e_repo = NULL;
 	int rc = 0;
 
 	struct gcli_fetch_list_ctx fl = {
@@ -79,14 +80,9 @@ gitlab_get_releases(struct gcli_ctx *ctx, char const *owner, char const *repo,
 
 	*list = (struct gcli_release_list) {0};
 
-	e_owner = gcli_urlencode(owner);
-	e_repo  = gcli_urlencode(repo);
-
-	url = sn_asprintf("%s/projects/%s%%2F%s/releases", gcli_get_apibase(ctx),
-	                  e_owner, e_repo);
-
-	free(e_owner);
-	free(e_repo);
+	rc = gitlab_repo_make_url(ctx, repo_path, &url, "/releases");
+	if (rc < 0)
+		return rc;
 
 	rc = gcli_fetch_list(ctx, url, &fl);
 
@@ -97,21 +93,21 @@ gitlab_get_releases(struct gcli_ctx *ctx, char const *owner, char const *repo,
 }
 
 int
-gitlab_create_release(struct gcli_ctx *ctx, struct gcli_new_release const *release)
+gitlab_create_release(struct gcli_ctx *ctx, struct gcli_create_release_args const *release)
 {
-	char *e_owner = NULL, *e_repo = NULL, *url = NULL, *payload = NULL;
+	char *url = NULL, *payload = NULL;
 	struct gcli_jsongen gen = {0};
 	int rc = 0;
 
 	/* Warnings because unsupported on gitlab */
 	if (release->prerelease)
-		warnx("prereleases are not supported on GitLab, option ignored");
+		gcli_warnx(ctx, "prereleases are not supported on GitLab, option ignored");
 
 	if (release->draft)
-		warnx("draft releases are not supported on GitLab, option ignored");
+		gcli_warnx(ctx, "draft releases are not supported on GitLab, option ignored");
 
 	if (release->assets_size)
-		warnx("GitLab release asset uploads are not yet supported");
+		gcli_warnx(ctx, "GitLab release asset uploads are not yet supported");
 
 	/* Payload generation */
 	gcli_jsongen_init(&gen);
@@ -140,44 +136,34 @@ gitlab_create_release(struct gcli_ctx *ctx, struct gcli_new_release const *relea
 	gcli_jsongen_free(&gen);
 
 	/* Generate URL */
-	e_owner = gcli_urlencode(release->owner);
-	e_repo = gcli_urlencode(release->repo);
+	rc = gitlab_repo_make_url(ctx, &release->repo_path, &url, "/releases");
 
-	/* https://docs.github.com/en/rest/reference/repos#create-a-release */
-	url = sn_asprintf("%s/projects/%s%%2F%s/releases", gcli_get_apibase(ctx),
-	                  e_owner, e_repo);
+	/* perform request */
+	if (rc == 0) {
+		rc = gcli_fetch_with_method(ctx, "POST", url, payload,
+		                            NULL, NULL);
+	}
 
-	free(e_owner);
-	free(e_repo);
-
-	rc = gcli_fetch_with_method(ctx, "POST", url, payload, NULL, NULL);
-
-	free(url);
-	free(payload);
+	gcli_clear_ptr(&url);
+	gcli_clear_ptr(&payload);
 
 	return rc;
 }
 
 int
-gitlab_delete_release(struct gcli_ctx *ctx, char const *owner,
-                      char const *repo, char const *id)
+gitlab_delete_release(struct gcli_ctx *ctx, struct gcli_path const *const path,
+                      char const *const id)
 {
-	char *url     = NULL;
-	char *e_owner = NULL;
-	char *e_repo  = NULL;
-	int   rc      = 0;
+	char *url = NULL;
+	int rc = 0;
 
-	e_owner = gcli_urlencode(owner);
-	e_repo  = gcli_urlencode(repo);
-
-	url = sn_asprintf("%s/projects/%s%%2F%s/releases/%s", gcli_get_apibase(ctx),
-	                  e_owner, e_repo, id);
+	rc = gitlab_repo_make_url(ctx, path, &url, "/releases/%s", id);
+	if (rc < 0)
+		return rc;
 
 	rc = gcli_fetch_with_method(ctx, "DELETE", url, NULL, NULL, NULL);
 
-	free(url);
-	free(e_owner);
-	free(e_repo);
+	gcli_clear_ptr(&url);
 
 	return rc;
 }
