@@ -1,5 +1,5 @@
 /*
- * Copyright 2023 Nico Sonack <nsonack@herrhotzenplotz.de>
+ * Copyright 2023-2025 Nico Sonack <nsonack@herrhotzenplotz.de>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -33,6 +33,7 @@
 
 #include <config.h>
 
+#include <assert.h>
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -65,8 +66,7 @@ gcli_jsongen_init(struct gcli_jsongen *gen)
 void
 gcli_jsongen_free(struct gcli_jsongen *gen)
 {
-	free(gen->buffer);
-	gen->buffer = NULL;
+	gcli_clear_ptr(&gen->buffer);
 	gen->buffer_size = 0;
 	gen->buffer_capacity = 0;
 
@@ -145,8 +145,10 @@ int
 gcli_jsongen_begin_object(struct gcli_jsongen *gen)
 {
 	/* Cannot put a json object into a json object key */
-	if (is_object_scope(gen) && !gen->await_object_value)
+	if (is_object_scope(gen) && !gen->await_object_value) {
+		assert(0 && "attempt to use json object as object key");
 		return -1;
+	}
 
 	put_comma_if_needed(gen);
 
@@ -163,8 +165,10 @@ gcli_jsongen_begin_object(struct gcli_jsongen *gen)
 int
 gcli_jsongen_end_object(struct gcli_jsongen *gen)
 {
-	if (pop_scope(gen) != GCLI_JSONGEN_OBJECT)
+	if (pop_scope(gen) != GCLI_JSONGEN_OBJECT) {
+		assert(0 && "unbalanced json scopes");
 		return -1;
+	}
 
 	append_str(gen, "}");
 
@@ -178,8 +182,10 @@ int
 gcli_jsongen_begin_array(struct gcli_jsongen *gen)
 {
 	/* Cannot put a json array into a json object key */
-	if (is_object_scope(gen) && !gen->await_object_value)
+	if (is_object_scope(gen) && !gen->await_object_value) {
+		assert(0 && "attempt to use array as object key");
 		return -1;
+	}
 
 	put_comma_if_needed(gen);
 
@@ -196,8 +202,10 @@ gcli_jsongen_begin_array(struct gcli_jsongen *gen)
 int
 gcli_jsongen_end_array(struct gcli_jsongen *gen)
 {
-	if (pop_scope(gen) != GCLI_JSONGEN_ARRAY)
+	if (pop_scope(gen) != GCLI_JSONGEN_ARRAY) {
+		assert(0 && "unbalanced json scopes");
 		return -1;
+	}
 
 	append_str(gen, "]");
 
@@ -235,18 +243,20 @@ append_strf(struct gcli_jsongen *gen, char const *const fmt, ...)
 int
 gcli_jsongen_objmember(struct gcli_jsongen *gen, char const *const key)
 {
-	if (!is_object_scope(gen))
+	if (!is_object_scope(gen)) {
+		assert(0 && "gcli_jsongen_objmember called outside object scope");
 		return -1;
+	}
 
 	put_comma_if_needed(gen);
-	char *const e_key = gcli_json_escape_cstr(key);
+	char *e_key = gcli_json_escape_cstr(key);
 
 	append_strf(gen, "\"%s\": ", e_key);
 
 	gen->first_elem = false;
 	gen->await_object_value = true;
 
-	free(e_key);
+	gcli_clear_ptr(&e_key);
 
 	return 0;
 }
@@ -292,14 +302,14 @@ int
 gcli_jsongen_string(struct gcli_jsongen *gen, char const *value)
 {
 	put_comma_if_needed(gen);
-	char *const e_value = gcli_json_escape_cstr(value);
+	char *e_value = gcli_json_escape_cstr(value);
 
 	append_strf(gen, "\"%s\"", e_value);
 
 	gen->await_object_value = false;
 	gen->first_elem = false;
 
-	free(e_value);
+	gcli_clear_ptr(&e_value);
 
 	return 0;
 }
