@@ -1,5 +1,5 @@
 /*
- * Copyright 2023 Nico Sonack <nsonack@herrhotzenplotz.de>
+ * Copyright 2023-2025 Nico Sonack <nsonack@herrhotzenplotz.de>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -31,50 +31,40 @@
 #include <gcli/milestones.h>
 
 int
-gcli_get_milestones(struct gcli_ctx *ctx, char const *const owner,
-                    char const *const repo, int const max,
-                    struct gcli_milestone_list *const out)
+gcli_get_milestones(struct gcli_ctx *ctx, struct gcli_path const *const path,
+                    int const max, struct gcli_milestone_list *const out)
 {
-	gcli_null_check_call(get_milestones, ctx, owner, repo, max, out);
+	gcli_null_check_call(get_milestones, ctx, path, max, out);
 }
 
 int
-gcli_get_milestone(struct gcli_ctx *ctx, char const *owner, char const *repo,
-                   gcli_id const milestone, struct gcli_milestone *const out)
+gcli_get_milestone(struct gcli_ctx *ctx, struct gcli_path const *const path,
+                   struct gcli_milestone *const out)
 {
-	gcli_null_check_call(get_milestone, ctx, owner, repo, milestone, out);
+	gcli_null_check_call(get_milestone, ctx, path, out);
 }
 
 int
 gcli_create_milestone(struct gcli_ctx *ctx,
+                      struct gcli_path const *const repo,
                       struct gcli_milestone_create_args const *args)
 {
-	gcli_null_check_call(create_milestone, ctx, args);
+	gcli_null_check_call(create_milestone, ctx, repo, args);
 }
 
 int
-gcli_delete_milestone(struct gcli_ctx *ctx, char const *const owner,
-                      char const *const repo, gcli_id const milestone)
+gcli_delete_milestone(struct gcli_ctx *ctx, struct gcli_path const *const path)
 {
-	gcli_null_check_call(delete_milestone, ctx, owner, repo, milestone);
+	gcli_null_check_call(delete_milestone, ctx, path);
 }
 
 void
 gcli_free_milestone(struct gcli_milestone *const it)
 {
-	free(it->title);
-	it->title = NULL;
-	free(it->state);
-	it->state = NULL;
-	free(it->created_at);
-	it->created_at = NULL;
-
-	free(it->description);
-	it->description = NULL;
-	free(it->updated_at);
-	it->updated_at = NULL;
-	free(it->due_date);
-	it->due_date = NULL;
+	gcli_clear_ptr(&it->title);
+	gcli_clear_ptr(&it->state);
+	gcli_clear_ptr(&it->description);
+	gcli_clear_ptr(&it->web_url);
 }
 
 void
@@ -83,25 +73,22 @@ gcli_free_milestones(struct gcli_milestone_list *const it)
 	for (size_t i = 0; i < it->milestones_size; ++i)
 		gcli_free_milestone(&it->milestones[i]);
 
-	free(it->milestones);
-	it->milestones = NULL;
+	gcli_clear_ptr(&it->milestones);
 	it->milestones_size = 0;
 }
 
 int
-gcli_milestone_get_issues(struct  gcli_ctx *ctx, char const *const owner,
-                          char const *const repo, gcli_id const milestone,
+gcli_milestone_get_issues(struct  gcli_ctx *ctx,
+                          struct gcli_path const *const milestone_path,
                           struct gcli_issue_list *const out)
 {
-	gcli_null_check_call(get_milestone_issues, ctx, owner, repo, milestone,
-	                     out);
+	gcli_null_check_call(get_milestone_issues, ctx, milestone_path, out);
 }
 
 int
-gcli_milestone_set_duedate(struct  gcli_ctx *ctx, char const *const owner,
-                           char const *const repo, gcli_id const milestone,
+gcli_milestone_set_duedate(struct  gcli_ctx *ctx,
+                           struct gcli_path const *const path,
                            char const *const date)
 {
-	gcli_null_check_call(milestone_set_duedate, ctx, owner, repo,
-	                     milestone, date);
+	gcli_null_check_call(milestone_set_duedate, ctx, path, date);
 }
