@@ -1,5 +1,5 @@
 /*
- * Copyright 2021, 2022 Nico Sonack <nsonack@herrhotzenplotz.de>
+ * Copyright 2021-2025 Nico Sonack <nsonack@herrhotzenplotz.de>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -44,11 +44,10 @@ gcli_get_repos(struct gcli_ctx *ctx, char const *owner, int const max,
 void
 gcli_repo_free(struct gcli_repo *it)
 {
-	free(it->full_name);
-	free(it->name);
-	free(it->owner);
-	free(it->date);
-	free(it->visibility);
+	gcli_clear_ptr(&it->full_name);
+	gcli_clear_ptr(&it->name);
+	gcli_clear_ptr(&it->owner);
+	gcli_clear_ptr(&it->visibility);
 	memset(it, 0, sizeof(*it));
 }
 
@@ -59,16 +58,14 @@ gcli_repos_free(struct gcli_repo_list *const list)
 		gcli_repo_free(&list->repos[i]);
 	}
 
-	free(list->repos);
-
-	list->repos = NULL;
+	gcli_clear_ptr(&list->repos);
 	list->repos_size = 0;
 }
 
 int
-gcli_repo_delete(struct gcli_ctx *ctx, char const *owner, char const *repo)
+gcli_repo_delete(struct gcli_ctx *ctx, struct gcli_path const *const path)
 {
-	gcli_null_check_call(repo_delete, ctx, owner, repo);
+	gcli_null_check_call(repo_delete, ctx, path);
 }
 
 int
@@ -79,8 +76,8 @@ gcli_repo_create(struct gcli_ctx *ctx, struct gcli_repo_create_options const *op
 }
 
 int
-gcli_repo_set_visibility(struct gcli_ctx *ctx, char const *const owner,
-                         char const *const repo, gcli_repo_visibility vis)
+gcli_repo_set_visibility(struct gcli_ctx *ctx, struct gcli_path const *const path,
+                         gcli_repo_visibility vis)
 {
-	gcli_null_check_call(repo_set_visibility, ctx, owner, repo, vis);
+	gcli_null_check_call(repo_set_visibility, ctx, path, vis);
 }
