@@ -1,5 +1,5 @@
 /*
- * Copyright 2023 Nico Sonack <nsonack@herrhotzenplotz.de>
+ * Copyright 2023-2025 Nico Sonack <nsonack@herrhotzenplotz.de>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -31,9 +31,10 @@
 #include <config.h>
 #endif
 
-#include <gcli/forges.h>
-#include <gcli/sshkeys.h>
 #include <gcli/cmd/table.h>
+#include <gcli/forges.h>
+#include <gcli/port/util.h>
+#include <gcli/sshkeys.h>
 
 int
 gcli_sshkeys_get_keys(struct gcli_ctx *ctx, struct gcli_sshkey_list *out)
@@ -45,14 +46,12 @@ void
 gcli_sshkeys_free_keys(struct gcli_sshkey_list *list)
 {
 	for (size_t i = 0; i < list->keys_size; ++i) {
-		free(list->keys[i].title);
-		free(list->keys[i].key);
-		free(list->keys[i].created_at);
+		gcli_clear_ptr(&list->keys[i].title);
+		gcli_clear_ptr(&list->keys[i].key);
 	}
 
-	free(list->keys);
+	gcli_clear_ptr(&list->keys);
 
-	list->keys = NULL;
 	list->keys_size = 0;
 }
 
@@ -68,12 +67,12 @@ gcli_sshkeys_add_key(struct gcli_ctx *ctx, char const *title,
 		return gcli_error(ctx, "ssh_add_key is not supported by this forge");
 	}
 
-	rc = sn_read_file(public_key_path, &buffer);
+	rc = gcli_read_file(public_key_path, &buffer);
 	if (rc < 0)
 		return rc;
 
 	rc = forge->add_sshkey(ctx, title, buffer, out);
-	free(buffer);
+	gcli_clear_ptr(&buffer);
 
 	return rc;
 }
